<?php
/**
 * @brief Client specific methods
 * @details Engine / Tool Library
 * @author CaMykS Team <camyks.contact@gmail.com>
 * @version 1.0.147
 * @date Creation: Jun 2007
 * @date Modification: Nov 2023
 * @copyright 2007 - 2023 CaMykS Team
 * @note This program is distributed as is - WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
global $softwareProperties;

/**
 * Return client IP address.
 * @return string
 */
function client_getIp() {
    if (!empty($_SERVER['HTTP_CLIENT_IP']))
        $ip = $_SERVER['HTTP_CLIENT_IP'];
    elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR']))
        $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
    elseif (!empty($_SERVER['HTTP_X_FORWARDED']))
        $ip = $_SERVER['HTTP_X_FORWARDED'];
    elseif (!empty($_SERVER['HTTP_FORWARDED_FOR']))
        $ip = $_SERVER['HTTP_FORWARDED_FOR'];
    elseif (!empty($_SERVER['HTTP_FORWARDED']))
        $ip = $_SERVER['HTTP_FORWARDED'];
    elseif (!empty($_SERVER['REMOTE_ADDR']))
        $ip = $_SERVER['REMOTE_ADDR'];
    else
        return '';

    if (string_isValidIPv4($ip))
        return $ip;
    return '';
}

/**
 * Return client user agent.
 * @return string
 */
function client_getUserAgent() {
    return (isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : '');
}

/**
 * Check if page is loaded in HTTPS (SSL).
 * @return boolean result
 */
function client_isHTTPS() {
    return (!empty($_SERVER['HTTPS']) or (isset($_SERVER['HTTP_X_FORWARDED_PROTO']) and $_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https'));
}

/**
 * Return client referer site.
 * @return string
 */
function client_getRefererSite() {
    return (isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : '');
}

/**
 * Check if the server is also the client.
 * @return boolean result
 */
function client_isServerClient() {
    return ($_SERVER['SERVER_ADDR'] == client_getIp());
}

/**
 * Check if client is marked as Do Not Track.
 * @return boolean result
 */
function client_getDoNotTrack() {
    return (isset($_SERVER['HTTP_DNT']) and $_SERVER['HTTP_DNT'] == 1);
}

/**
 * Return client informations.
 * @param mixte $ua
 * @return array
 */
function client_getUAInfos($ua=false) {
    $req = true;

    /* get user agent */
    if ($ua===false) $ua = client_getUserAgent();

    /*** check fake visitors */
    if (($c = client_getFakeInfosFromUserAgent($ua)) !== false)
        return $c;

    /*** check hacks */
    if (($c = client_getHackInfosFromUserAgent($ua)) !== false)
        return $c;

    /*** check unknown */
    if (($c = client_getUnknownInfosFromUserAgent($ua)) !== false)
        return $c;

    /*** check before bots */
    if (($c = client_getBeforeBotInfosFromUserAgent($ua)) !== false)
        return $c;

    /*** check tool */
    if (($c = client_getToolInfosFromUserAgent($ua)) !== false)
        return $c;

    /*** check bot */
    if (($c = client_getBotInfosFromUserAgent($ua)) !== false)
        return $c;

    /*** check generic bot/tool */
    if (($c = client_getGenericInfosFromUserAgent($ua)) !== false)
        return $c;

    /*** check plateform ***/

    /* init result value */
    $infos = array();

    /* Load max versions to check and default fake value */
    $fake = array('platform_type'     => 'fake');

    /* Check Playstation 4 / Vita */
    if (preg_match('/PlayStation (.*)(\)?;)?[ |\/]([0-9\.]*)\)/Ui', $ua, $matches)) {
        $model = 'PlayStation '.$matches[1];
        include_once('clientDevices/Sony.php.inc');
        return array_merge($infos, get_SonyClientDevice($model), array(
            'system_title'      => $model,
            'system_version'    => $matches[3],
            'browser_title'     => $model.' Browser',
            'browser_version'   => '--',
        ));
    }
    /* Check Black Berry */
    if (preg_match('/BlackBerry([^\/;]{1,})\/([0-9\.]*)/', $ua, $matches))
        return _client_mergeUAInfos($req, array(
            'platform_company'  => 'BlackBerry',
            'platform_model'    => $matches[1],
            'platform_type'     => 'handheld',
            'platform_format'   => 'handheld',
            'system_title'      => 'BlackBerry OS',
            'system_version'    => $matches[2],
            'browser_title'     => 'BB Browser',
            'browser_version'   => '--',
        ));
    if (preg_match('/BlackBerry ([^;]{1,});?.*Version\/([0-9\.]*) /', $ua, $matches))
        return _client_mergeUAInfos($req, array(
            'platform_company'  => 'BlackBerry',
            'platform_model'    => $matches[1],
            'platform_type'     => 'handheld',
            'platform_format'   => 'handheld',
            'system_title'      => 'BlackBerry OS',
            'system_version'    => $matches[2],
            'browser_title'     => 'BB Browser',
            'browser_version'   => '--',
        ));
    if (preg_match('/BB([^\/;]{1,});/', $ua, $matches)) {
        if (preg_match('/ Version\/([0-9\.]*) /', $ua, $matches2))
            $version = $matches2[1];
        else
            $version = '--';
        return _client_mergeUAInfos($req, array(
            'platform_company'  => 'BlackBerry',
            'platform_model'    => $matches[1],
            'platform_type'     => 'handheld',
            'platform_format'   => 'handheld',
            'system_title'      => 'BlackBerry OS',
            'system_version'    => $matches[1],
            'browser_title'     => 'BB Browser',
            'browser_version'   => $version,
        ));
    }
    else if (preg_match('/ (BB|ST)([ ]?[A-Z][0-9\-]*)( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/BlackBerry.php.inc');
        $device = $matches[1].$matches[2];
        $infos = array_merge($infos, get_BlackBerryClientDevice($device));
    }

    /* Check Nintendo - before iPhone */
    elseif (preg_match('/Nintendo ([^ \/;\)]+)/', $ua, $matches)) {
        $infos = array_merge($infos, array(
            'platform_company'  => 'Nintendo',
            'platform_model'    => $matches[1],
            'platform_format'   => preg_match('/DS/', $matches[1])?'handheld':'desktop',
            'platform_type'     => preg_match('/DS/', $matches[1])?'handheld':'computer',
        ));
    }

    /* Check Nokia - before iPhone */
    elseif (preg_match('/RX-51 N900/', $ua, $matches))
        $infos = array_merge($infos, array(
            'platform_company'  => 'Nokia',
            'platform_model'    => 'N900',
            'platform_type'     => 'handheld',
            'platform_format'   => 'handheld',
            'system_title'      => 'Maemo',
            'system_version'    => 5,
        ));
    elseif (preg_match('/Nokia(?: |_|\-|\/|)([^_\-\/]+)( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Nokia.php.inc');
        $infos = array_merge($infos, get_NokiaClientDevice(trim($matches[1])));
    } elseif (preg_match('/(Lumia [a-zA-Z0-9 ]*)( Build|\)|;)/', $ua, $matches)) {
        include_once('clientDevices/Nokia.php.inc');
        $infos = array_merge($infos, get_NokiaClientDevice($matches[1]));
    } elseif (preg_match('/(TA\-[0-9]{4})( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Nokia.php.inc');
        $infos = array_merge($infos, get_NokiaClientDevice($matches[1]));
    }

    /* Check Accent devices : before Archos */
    elseif (preg_match('/ Accent ([^\)]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Accent.php.inc');
        $model = strtolower(trim($matches[1]));
        $infos = array_merge($infos, get_AccentClientDevice($model));
    }

    /* Check Archos devices - before iOS */
    elseif (preg_match('/ Archos[;]? ([A-Za-z0-9\- ]{3,})( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/Archos.php.inc');
        $infos = array_merge($infos, get_ArchosClientDevice(trim($matches[1])));
    } elseif (preg_match('/ (A[C]?[0-9][05]B?)(HE|TI)( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Archos.php.inc');
        $device = trim($matches[1].$matches[2]);
        $infos = array_merge($infos, get_ArchosClientDevice($device));
    }

    /* Check iphone / ipod / ipad */
    elseif (preg_match('/(iPod|iPhone|iPad);?.*; CPU ([A-Za-z\ ]*)? ([0-9_]*)/', $ua, $matches)) {
        /* Check version */
        $version = str_replace('_', '.', $matches[3]);

        $infos = array_merge($infos, array(
            'platform_company'  => 'Apple',
            'platform_model'    => $matches[1],
            'platform_type'     => $matches[1]=='iPad'? 'tablet': 'handheld',
            'platform_format'   => $matches[1]=='iPad'? 'tablet': 'handheld',
            'system_title'      => 'iOS',
            'system_version'    => $version,
        ));

        /* Check for safari mobile version */
        if (preg_match('/Version\/([0-9\.]*)/', $ua, $matches)) {
            $infos = array_merge($infos, array(
                'browser_title'     => 'Safari Mobile',
                'browser_version'   => $matches[1],
            ));
        }

        /* Check for device model */
        if (preg_match('/(iPhone|iPod|iPad)([0-9_\,]{3,})/i', $ua, $matches)) {
            include_once('clientDevices/Apple.php.inc');
            $model = $matches[1].str_replace(array('_', ','), '.', $matches[2]);
            $infos = array_merge($infos, get_AppleClientDevice($model));
        }

    } elseif (preg_match('/(iPhone|iPod|iPad)\/([0-9\.]*) hw\/(iPhone|iPod|iPad)([0-9_]{3,})/i', $ua, $matches)) {
        /* Check version */
        $version = str_replace('_', '.', $matches[2]);

        $infos = array_merge($infos, array(
            'platform_company'  => 'Apple',
            'platform_model'    => $matches[1],
            'platform_type'     => $matches[1]=='iPad'? 'tablet': 'handheld',
            'platform_format'   => $matches[1]=='iPad'? 'tablet': 'handheld',
            'system_title'      => 'iOS',
            'system_version'    => $version,
        ));
        include_once('clientDevices/Apple.php.inc');
        $model = $matches[3].str_replace(array('_', ','), '.', $matches[4]);
        $infos = array_merge($infos, get_AppleClientDevice($model));

    } elseif (preg_match('/(iPod|iPhone|iPad)([0-9_\,\.]{3,})?/', $ua, $matches)) {
        $infos = array_merge($infos, array(
            'platform_company'  => 'Apple',
            'platform_model'    => $matches[1],
            'platform_type'     => $matches[1]=='iPad'? 'tablet': 'handheld',
            'platform_format'   => $matches[1]=='iPad'? 'tablet': 'handheld',
            'system_title'      => 'iOS',
        ));

        /* Check for device model */
        if (isset($matches[2])) {
            include_once('clientDevices/Apple.php.inc');
            $model = $matches[1].str_replace(array('_', ','), '.', $matches[2]);
            $infos = array_merge($infos, get_AppleClientDevice($model));
        }

        /* Check for iOS version */
        if (preg_match('/iOS ([0-9_\,\.]{3,})/', $ua, $matches)) {
            $infos['system_version'] = str_replace(array('_', ','), '.', $matches[1]);
        }

        /* Check for darwin version */
        if (preg_match('/ Darwin\/([0-9\.]*)(?!.*\(x86_64\))/i', $ua, $matches)) {
            /* Check version */
            $version = $matches[1];
            $version = explode('.', $version);
            $version = (int)$version[0]-6;

            /* build info */
            $infos['system_version'] = $version;
        }

    } elseif (preg_match('/ Darwin\/([0-9\.]*)(?!.*\(x86_64\))/i', $ua, $matches)) {
        /* Check version */
        $version = $matches[1];
        $version = explode('.', $version);
        $version = (int)$version[0]-6;

        /* build info */
        $infos = array_merge($infos, array(
            'platform_company'  => 'Apple',
            'platform_model'    => 'iPhone/iPad/iPod',
            'platform_type'     => 'handheld',
            'platform_format'   => 'handheld',
            'system_title'      => 'iOS',
            'system_version'    => $version
        ));
    }

    /* Check Google */
    elseif (preg_match('/ (Nexus|Pixel)([^\/;]+)?( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Google.php.inc');
        $model = $matches[1].(isset($matches[2])?$matches[2]:'');
        $infos = array_merge($infos, get_GoogleClientDevice($model));
    }

    /* Check HTC */
    elseif (preg_match('/HTC;([A-Za-z0-9 ]*)/', $ua, $matches)) {
        include_once('clientDevices/HTC.php.inc');
        $infos = array_merge($infos, get_HTCClientDevice($matches[1]));
    } elseif (preg_match('/HTC[ |_]([^\/]*)( Build|\-|\))/i', $ua, $matches)) {
        include_once('clientDevices/HTC.php.inc');
        $model = preg_replace('/_/', ' ', $matches[1]);
        $infos = array_merge($infos, get_HTCClientDevice($model));
    } elseif (preg_match('/HTC([A-Za-z0-9]+)/', $ua, $matches)) {
        include_once('clientDevices/HTC.php.inc');
        $model = preg_replace('/_/', ' ', $matches[1]);
        $infos = array_merge($infos, get_HTCClientDevice($model));
    } elseif (preg_match('/ (ADR6300|2PYB2|2PQ93)/', $ua, $matches)) {
        include_once('clientDevices/HTC.php.inc');
        $infos = array_merge($infos, get_HTCClientDevice($matches[1]));
    } elseif (preg_match('/ (dream)\)/', $ua, $matches)) {
        include_once('clientDevices/HTC.php.inc');
        $infos = array_merge($infos, get_HTCClientDevice($matches[1]));
    }

    /* Check LG */
    elseif (preg_match('/^LG(?:E|)(?: |_|\-|\/)([^ _\-\/]+)/', $ua, $matches)) {
        include_once('clientDevices/LG.php.inc');
        $infos = array_merge($infos, get_LGClientDevice($matches[1]));
    } elseif (preg_match('/; L[G|M]-?([A-Z]{1,2}[0-9]{3}\.?[A-Z]{0,})/', $ua, $matches) or preg_match('/; LG([A-Z]{1,2}[0-9]*[A-Z]{0,})/', $ua, $matches)) {
        include_once('clientDevices/LG.php.inc');
        $infos = array_merge($infos, get_LGClientDevice($matches[1]));
    } elseif (preg_match('/; (VS[0-9]{3})( 4G)?( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/LG.php.inc');
        $infos = array_merge($infos, get_LGClientDevice($matches[1]));
    } elseif (preg_match('/; (VK810 4G)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/LG.php.inc');
        $infos = array_merge($infos, get_LGClientDevice($matches[1]));
    }

    /* Check Motorola */
    elseif (preg_match('/mot(?:-|orola|o|\-moto)([^\/;]+)( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/Motorola.php.inc');
        $infos = array_merge($infos, get_MotorolaClientDevice(trim($matches[1])));
    } elseif (preg_match('/ (Droid|MB200|RAZR HD|Milestone)/', $ua, $matches)) {
        include_once('clientDevices/Motorola.php.inc');
        $infos = array_merge($infos, get_MotorolaClientDevice($matches[1]));
    } elseif (preg_match('/XT([0-9]{3,})/', $ua, $matches)) {
        include_once('clientDevices/Motorola.php.inc');
        $infos = array_merge($infos, get_MotorolaClientDevice($matches[0]));
    }

    /* Check Palm devices */
    elseif (preg_match('/(Pre|Pixi)\/[0-9\.]*$/', $ua, $matches))
        $infos = array_merge($infos, array(
            'platform_company'  => 'Palm',
            'platform_model'    => $matches[1],
            'platform_type'     => 'handheld',
            'platform_format'   => 'handheld',
        ));

    /* Check Huawei - before Sony & Samsung */
    elseif (($device = _client_isHuaweiDevice($ua)) !== false) {
        $infos = array_merge($infos, get_HuaweiClientDevice($device));
    }

    /* Check BQ devices, before Samsung */
    elseif (preg_match('/ (Aquaris [A-Za-z0-9 \.]*)( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/BQ.php.inc');
        $infos = array_merge($infos, get_BQClientDevice($matches[1]));
    } elseif (preg_match('/ BQ(ru|S)?(-|\_| )?([0-9]{4}[A-Za-z]*|Aquaris [A-Z0-9 \.]*)( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/BQ.php.inc');
        $infos = array_merge($infos, get_BQClientDevice($matches[3]));
    }

    /* Check Samsung devices */
    elseif (preg_match('/(GT|SM|SGH|SHV|SC|SPH)-([A-Z0-9]{3,})/', $ua, $matches)) {
        include_once('clientDevices/Samsung.php.inc');
        $infos = array_merge($infos, get_SamsungClientDevice($matches[0]));
    } elseif (preg_match('/; (SCV[0-9]{2})( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Samsung.php.inc');
        $infos = array_merge($infos, get_SamsungClientDevice($matches[1]));
    } elseif (preg_match('/SAMSUNG\-(?:SGH\-|)([^\/]+)/', $ua, $matches)) {
        $infos = array_merge($infos, array(
            'platform_company'  => 'Samsung',
            'platform_model'    => $matches[1],
            'platform_type'     => 'handheld',
            'platform_format'   => 'handheld',
        ));
    } elseif (preg_match('/ (Galaxy|E[0-9]) Build/', $ua, $matches)) {
        include_once('clientDevices/Samsung.php.inc');
        $infos = array_merge($infos, get_SamsungClientDevice($matches[1]));
    } elseif (preg_match('/(klte)/', $ua, $matches))
        $infos = array_merge($infos, array(
            'platform_company'  => 'Samsung',
            'platform_model'    => 'Galaxy S5',
            'platform_type'     => 'handheld',
            'platform_format'   => 'handheld',
        ));

    /* Check Samsung PC */
    elseif (preg_match('/MASM|MASMJS|SMJB/', $ua, $matches))
        $infos = array_merge($infos, array(
            'platform_company'  => 'Samsung (PC)'
        ));

    /* Check Lenovo devices - before Huawei */
    elseif (preg_match('/ (IdeaTab|Yoga|Lenovo|Zuk)[ |_]([A-Z0-9\- _]{2,})( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Lenovo.php.inc');
        $infos = array_merge($infos, get_LenovoClientDevice(trim($matches[2])));
    }
    elseif (preg_match('/ (Z2 Plus|Tab2A710F)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Lenovo.php.inc');
        $infos = array_merge($infos, get_LenovoClientDevice(trim($matches[1])));
    }

    /* Check Lenovo PCs */
    elseif (preg_match('/MALC|MALCJS|MALEJS|MALN|MALNJS|LCJB|LEN2|LCTE/', $ua, $matches))
        $infos = array_merge($infos, array(
            'platform_company'  =>'Lenovo (PC)',
        ));

    /* Check Kyocera - before Sony */
    elseif (($device = _client_isKyoceraDevice($ua)) !== false) {
        $infos = array_merge($infos, get_KyoceraClientDevice($device));
    }

    /* Check OnePlus - to be inserted before Sony */
    elseif (($device = _client_isOnePlusDevice($ua)) !== false) {
        $infos = array_merge($infos, get_OnePlusClientDevice($device));
    }

    /* Check SonyEricsson */
    elseif (preg_match('/SonyEricsson([^ \/]+)/', $ua, $matches)) {
        include_once('clientDevices/SonyEricsson.php.inc');
        $infos = array_merge($infos, get_SonyEricssonClientDevice($matches[1]));
    }

    /* Check Sony mobiles and tablets */
    elseif (preg_match('/ ([C-J][0-9]{4})( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Sony.php.inc');
        $infos = array_merge($infos, get_SonyClientDevice($matches[1]));
    }
    elseif (preg_match('/SO(V[34][0-9]|-[0-9]{2}[A-Z])/', $ua, $matches)) {
        include_once('clientDevices/Sony.php.inc');
        $infos = array_merge($infos, get_SonyClientDevice($matches[0]));
    }
    elseif (preg_match('/ (SGP[0-9a-zA-Z]{3,})( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Sony.php.inc');
        $infos = array_merge($infos, get_SonyClientDevice($matches[1]));
    }
    elseif (preg_match('/; (XQ-[0-9a-zA-Z]{4})( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Sony.php.inc');
        $infos = array_merge($infos, get_SonyClientDevice($matches[1]));
    }
    elseif (preg_match('/; (Xperia)(.*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Sony.php.inc');
        $infos = array_merge($infos, get_SonyClientDevice($matches[1].$matches[2]));
    }
    elseif (preg_match('/ (LT28i|502SO)( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Sony.php.inc');
        $infos = array_merge($infos, get_SonyClientDevice($matches[1]));
    }

    /* Check Sony PCs */
    elseif (preg_match('/MASE|MASEJS|MASP|MASPJS|MASA|MASAJS/', $ua, $matches))
        $infos = array_merge($infos, array(
            'platform_company'  =>'Sony (PC)',
        ));

    /* Check Spice Mobility devices */
    elseif (preg_match('/Spice ([A-Z0-9\-]{4,})/i', $ua, $matches)) {
        include_once('clientDevices/SpiceMobility.php.inc');
        $model = trim($matches[1]);
        $infos = array_merge($infos, get_SpiceMobilityClientDevice($model));
    }

    /* Check Sumvier devices - before Oukitel */
    elseif (preg_match('/; (K41)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Sumvier.php.inc');
        $model = trim($matches[1]);
        $infos = array_merge($infos, get_SumvierClientDevice($model));
    }

    /* Check Outikel mobiles */
    elseif (preg_match('/; (K[0-9]{1,4})([A-zA-Z]*)?( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Oukitel.php.inc');
        $infos = array_merge($infos, get_OukitelClientDevice(trim($matches[1].$matches[2])));
    } elseif (preg_match('/; (WP[125]|C1[57] Pro|U20 Plus)( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Oukitel.php.inc');
        $infos = array_merge($infos, get_OukitelClientDevice($matches[1]));
    }

    /* Check Energizer mobiles - before Asus */
    elseif (preg_match('/ (P[0-9]{3}S)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Energizer.php.inc');
        $infos = array_merge($infos, get_EnergizerClientDevice(trim($matches[1])));
    } elseif (preg_match('/ (PowerMaxP[0-9]{3}S)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Energizer.php.inc');
        $infos = array_merge($infos, get_EnergizerClientDevice(trim($matches[1])));
    } elseif (preg_match('/ Energy[ \-_]?([A-Z]?[0-9]{3}[A-Z]?)(LTE)?( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Energizer.php.inc');
        $model = trim($matches[1].$matches[2]);
        $infos = array_merge($infos, get_EnergizerClientDevice($model));
    }

    /* Check Asus mobiles and tablets */
    elseif (preg_match('/; (K|ME|P)([0-9]{2})([A-Z0-9]{1,2})( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Asus.php.inc');
        $infos = array_merge($infos, get_AsusClientDevice(trim($matches[1].$matches[2].$matches[3])));
    } elseif (preg_match('/; (Z?[A-Z][0-9]{3}[C|K|T]{1}L) Build/', $ua, $matches)) {
        include_once('clientDevices/Asus.php.inc');
        $infos = array_merge($infos, get_AsusClientDevice(trim($matches[1])));
    } elseif (preg_match('/; ASUS_([A-Z0-9]+)( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Asus.php.inc');
        $infos = array_merge($infos, get_AsusClientDevice(trim($matches[1])));
    } elseif (preg_match('/; (Zenfone .*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Asus.php.inc');
        $infos = array_merge($infos, get_AsusClientDevice(trim($matches[1])));
    } elseif (preg_match('/; Transformer ([A-Z0-9]+)( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Asus.php.inc');
        $infos = array_merge($infos, get_AsusClientDevice(trim($matches[1])));
    }

    /* Check Asus PCs */
    elseif (preg_match('/MAAU|ASJB|ASU2JS|ASU2|NP0[6789]|ASTE/', $ua, $matches))
        $infos = array_merge($infos, array(
            'platform_company'  => 'Asus (PC)',
        ));

    /* Check Micromax */
    elseif (preg_match('/Micromax(|_)([^ \/\)]+)/', $ua, $matches)) {
        include_once('clientDevices/Micromax.php.inc');
        $infos = array_merge($infos, get_MicromaxClientDevice($matches[2]));
    }

    /* Check ITel */
    elseif (preg_match('/ itel[ |_]([^\)\/]*)( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Itel.php.inc');
        $infos = array_merge($infos, get_ItelClientDevice($matches[1]));
    }

    /* Check Xbox / XBox One */
    elseif (preg_match('/Xbox(; Xbox [a-zA-Z0-9]*)?/i', $ua, $matches))
        $infos = array_merge($infos, array(
            'platform_company'  => 'Microsoft',
            'platform_model'    => isset($matches[1])?substr($matches[1], 2):'Xbox 360',
            'platform_type'     => 'computer',
            'platform_format'   => 'desktop',
        ));
    /* Check Xbox One, icons loader */
    elseif (preg_match('/\(Windows Phone 10.0; Android 6.0.1\)/', $ua, $matches))
        $infos = array_merge($infos, array(
            'platform_company'  => 'Microsoft',
            'platform_model'    => 'Xbox One',
            'platform_type'     => 'computer',
            'platform_format'   => 'desktop',
            'system_title'      => 'Windows',
            'system_version'    => '10',
        ));

    /* Check Amazon Kindle */
    elseif (preg_match('/ Kindle\/?([0-9\.]*)/', $ua, $matches))
        /* build info */
        $infos = array_merge($infos, array(
            'platform_company'  => 'Amazon',
            'platform_model'    => 'Kindle',
            'platform_type'     => 'tablet',
            'platform_format'   => 'tablet',
            'browser_title'     => 'Kindle browser',
            'browser_version'   => $matches[1],
        ));
    elseif (preg_match('/ (KF[A-Z]{2,4})( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Amazon.php.inc');
        $infos = array_merge($infos, get_AmazonClientDevice($matches[1]));
    } elseif (preg_match('/ (SD4930UR)( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Amazon.php.inc');
        $infos = array_merge($infos, get_AmazonClientDevice($matches[1]));
    }

    /* Check HiSense devices : before Acer */
    elseif (preg_match('/(Hisense |HS-)([A-Z0-9a-z _]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/HiSense.php.inc');
        $model = preg_replace('/_/', ' ', $matches[2]);
        $infos = array_merge($infos, get_HiSenseClientDevice($model));
    }

    /* Check Cubot devices : before Acer */
    elseif (preg_match('/ Cubot[ _\-]?([A-Za-z0-9\+ _]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Cubot.php.inc');
        $model = trim(preg_replace('/_/', ' ', $matches[1]));
        $infos = array_merge($infos, get_CubotClientDevice($model));
    } elseif (preg_match('/; (X[0-9]{1,2})( PRO| PLUS)( Build|\))/U', $ua, $matches)) {
        /* No case insentive to avoid matching x64 versions of Windows */
        include_once('clientDevices/Cubot.php.inc');
        $model = trim($matches[1].$matches[2]);
        $infos = array_merge($infos, get_CubotClientDevice($model));
    } elseif (preg_match('/; (GT90|KING_KONG_3|MAX 2)( Build|\))/U', $ua, $matches)) {
        /* No case insentive to avoid matching x64 versions of Windows */
        include_once('clientDevices/Cubot.php.inc');
        $model = trim(preg_replace('/_/', ' ', $matches[1]));
        $infos = array_merge($infos, get_CubotClientDevice($model));
    }

    /* Check Caterpillar mobiles - before Wiko, Acer */
    elseif (preg_match('/ (S[346][0-9])( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Cat.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_CatClientDevice($model));
    } elseif (preg_match('/; CAT([^.]*)( Build|\)|;)/Ui', $ua, $matches)) {
        include_once('clientDevices/Cat.php.inc');
        $model = trim($matches[1]);
        $infos = array_merge($infos, get_CatClientDevice($model));
    }

    /* Check Doogee devices, to be inserted before Xiaomi/Wiko/Acer/Umidigi */
    elseif (preg_match('/; (MIX(|Lite|2| 3)|N20(Pro)?|S90|S60Lite|S59|S[58]8Pro|S40Pro|S55_Lite|X[37]0|X[0-9]0L|X95|X95Pro|X5max_PRO)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Doogee.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_DoogeeClientDevice($model));
    } elseif (preg_match('/; (BL[0-9]{1,2}000)( PRO)?( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Doogee.php.inc');
        $model = $matches[1].$matches[2];
        $infos = array_merge($infos, get_DoogeeClientDevice($model));
    }

    /* Check Klipad devices : before Acer */
    elseif (preg_match('/; KLIPAD_(.*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Klipad.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_KlipadClientDevice($device));
    } elseif (preg_match('/; (V356)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Klipad.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_KlipadClientDevice($device));
    } elseif (preg_match('/; (KL[0-9]{3})( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Klipad.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_KlipadClientDevice($device));
    }

    /* Check Nomu devices - before Wiko */
    elseif (preg_match('/ (s10 pro|s30mini)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Nomu.php.inc');
        $device = strtolower($matches[1]);
        $infos = array_merge($infos, get_NomuClientDevice($device));
    }

    /* Check Sprint devices - before Memup */
    elseif (preg_match('/; Sprint (.*)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Sprint.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_SprintClientDevice($device));
    }

    /* Check Memup devices - before Wiko */
    elseif (preg_match('/; (SP|SlidePad )([^;\-\)]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Memup.php.inc');
        $model = $matches[2];
        $infos = array_merge($infos, get_MemupClientDevice($model));
    }

    /* Check Altice devices - before Wiko */
    elseif (preg_match('/; ALTICE_(.*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Altice.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_AlticeClientDevice($model));
    } elseif (preg_match('/; Star(xtrem|trail|addict|naute)([0-9 ]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Altice.php.inc');
        $model = 'Star'.$matches[1].trim($matches[2]);
        $infos = array_merge($infos, get_AlticeClientDevice($model));
    } elseif (preg_match('/; (SX41)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Altice.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_AlticeClientDevice($model));
    }

    /* Check Advan devices - before Wiko */
    elseif (preg_match('/ (50[0-9]{2}|S5E_NXT|S50[A-Z])( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Advan.php.inc');
        $infos = array_merge($infos, get_AdvanClientDevice($matches[1]));
    } elseif (preg_match('/ (i5K)( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Advan.php.inc');
        $infos = array_merge($infos, get_AdvanClientDevice($matches[1]));
    }

    /* Check SpreadTrum */
    elseif (preg_match('/SpreadTrum/', $ua, $matches))
        $infos = array_merge($infos, array(
            'platform_company'  => 'SpreadTrum',
            'platform_model'    => '',
            'platform_type'     => 'handheld',
            'platform_format'   => 'handheld',
        ));

    /* Check Maximus devices */
    elseif (preg_match('/ (MAX908)( Build|\)|;)/', $ua, $matches)) {
        include_once('clientDevices/Maximus.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_MaximusClientDevice($model));
    }

    /* Check Alcatel devices */
     elseif (preg_match('/; ([0-9]{4}[A-Z])(_EEA)?( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Alcatel.php.inc');
        $infos = array_merge($infos, get_AlcatelClientDevice($matches[1]));
    } elseif (preg_match('/; Alcatel[_| ](.*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Alcatel.php.inc');
        $infos = array_merge($infos, get_AlcatelClientDevice($matches[1]));
    } elseif (preg_match('/ (Orange KLIF|A460G)( Build|\)|;)/', $ua, $matches)) {
        include_once('clientDevices/Alcatel.php.inc');
        $infos = array_merge($infos, get_AlcatelClientDevice($matches[1]));
    }

    /* Check Lava Iris devices */
    elseif (preg_match('/ (Iris[A-Z0-9\+]+|Z50|A48_8GB|K1plus)( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/Lava.php.inc');
        $model = strtolower($matches[1]);
        $infos = array_merge($infos, get_LavaClientDevice($model));
    }

    /* Check Tecno devices */
    elseif (preg_match('/; Tecno[ |\-|_]?([A-Za-z0-9 ]+)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Tecno.php.inc');
        $device = trim($matches[1]);
        $infos = array_merge($infos, get_TecnoClientDevice($device));
    } elseif (preg_match('/; (Phantom6-Plus)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Tecno.php.inc');
        $device = trim($matches[1]);
        $infos = array_merge($infos, get_TecnoClientDevice($device));
    }

    /* Check Yezz Andy devices */
    elseif (preg_match('/ (Andy|Yezz)[_| |-]([A-Z0-9\._ ]{2,})( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Yezz.php.inc');
        $device = preg_replace('/_/', ' ', $matches[2]);
        $infos = array_merge($infos, get_YezzClientDevice($device));
    }

    /* Check Wing devices */
    elseif (preg_match('/ Wing_([A-Za-z0-9]{2,})( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/Wing.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_WingClientDevice($model));
    }

    /* Check Karbonn devices : before Leagoo */
    elseif (preg_match('/ Karbonn(|_)([A-Z][0-9]{2}) Build/i', $ua, $matches)) {
        include_once('clientDevices/Karbonn.php.inc');
        $infos = array_merge($infos, get_KarbonnClientDevice($matches[2]));
    } elseif (preg_match('/ (Aura|Alfa) ([A-Z][^\)\/]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Karbonn.php.inc');
        $infos = array_merge($infos, get_KarbonnClientDevice($matches[1].' '.$matches[2]));
    }

    /* Check Leagoo devices */
    elseif (preg_match('/(Leagoo_| )(Elite|Alfa|Kiicaa|Lead)[ |_|-]?([^\)\/]*)( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/Leagoo.php.inc');
        $infos = array_merge($infos, get_LeagooClientDevice($matches[2].' '.$matches[3]));
    } elseif (preg_match('/; (S8_Pro|Z6)( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/Leagoo.php.inc');
        $infos = array_merge($infos, get_LeagooClientDevice($matches[1]));
    }

    /* Check Wortmann devices */
    elseif (preg_match('/ (TERRA )?(PAD ?)(100[0-9])( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Wortmann.php.inc');
        $model = $matches[3];
        $infos = array_merge($infos, get_WortmannClientDevice($model));
    }

    /* Check QMobile devices */
    elseif (preg_match('/; (A3|LT700)( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/QMobile.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_QMobileClientDevice($model));
    }

    /* Check Haier devices */
    elseif (preg_match('/ (CDP7TAB4C8|G30) /', $ua, $matches)) {
        include_once('clientDevices/Haier.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_HaierClientDevice($model));
    } elseif (preg_match('/ (HM-[A-Z][0-9]{3}-FL)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Haier.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_HaierClientDevice($model));
    }

    /* Check Bloom */
    elseif (preg_match('/BLOOM Build/', $ua, $matches))
        $infos = array_merge($infos, array(
            'platform_company'  => 'Bloom',
            'platform_model'    => '',
            'platform_type'     => 'handheld',
            'platform_format'   => 'handheld',
        ));

    /* Check Carrefour */
    elseif (preg_match('/ (C[M|T][A-Z0-9]{3,4}[A-Z]?)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Carrefour.php.inc');
        $infos = array_merge($infos, get_CarrefourClientDevice($matches[1]));
    }

    /* Check Orange */
    elseif (preg_match('/ (Nura|Dive) ([^.]*)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Orange.php.inc');
        $device = $matches[1].' '.$matches[2];
        $infos = array_merge($infos, get_OrangeClientDevice($device));
    } elseif (preg_match('/Orange[ |\-|_]([^.]*)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Orange.php.inc');
        $infos = array_merge($infos, get_OrangeClientDevice($matches[1]));
    }

    /* Check Coolpad devices */
    elseif (preg_match('/; CP([0-9][A-Z0-9_\-]+)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Coolpad.php.inc');
        $infos = array_merge($infos, get_CoolpadClientDevice($matches[1]));
    } elseif (preg_match('/; Coolpad ([A-Z0-9_\-]+)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Coolpad.php.inc');
        $infos = array_merge($infos, get_CoolpadClientDevice($matches[1]));
    }

    /* Check Vestel devices */
    elseif (preg_match('/; (Venus.*)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Vestel.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_VestelClientDevice($model));
    }

    /* Check Hewlett Packard OEM computers */
    elseif (preg_match('/HPCMHP|HPNTDF|HPNTDFJS|HPDTDF|HPDTDFJS/', $ua, $matches)) {
        $infos = array_merge($infos, array(
            'platform_company'  => 'Hewlett Packard (PC)',
        ));
    }

    /* Check Dell OEM computers */
    elseif (preg_match('/MDDR|MDDRJS|MDDC|MDDCJS|MDDS|MDDSJS/', $ua, $matches)) {
        $infos = array_merge($infos, array(
            'platform_company'  => 'Dell (PC)',
        ));
    }

    /* Check Toshiba OEM computers */
    elseif (preg_match('/MATM|MATMJS|MATB|MATBJS|MATP|MATPJS|TNJB|TAJB/', $ua, $matches)) {
        $infos = array_merge($infos, array(
            'platform_company'  => 'Toshiba (PC)',
        ));
    }

    /* Check Acer OEM computers */
    elseif (preg_match('/MAAR|MAARJS/', $ua, $matches)) {
        $infos = array_merge($infos, array(
            'platform_company'  =>'Acer (PC)',
        ));
    }

    /* Check Medion OEM computers */
    elseif (preg_match('/MAMD/', $ua, $matches)) {
        $infos = array_merge($infos, array(
            'platform_company'  =>'Medion (PC)',
        ));
    }

    /* Check MSI OEM computers */
    elseif (preg_match('/MAMI|MAMIJS|MAM3/', $ua, $matches)) {
        $infos = array_merge($infos, array(
            'platform_company'  => 'Medion (PC)',
        ));
    }

    /* Check Gateway OEM computers */
    elseif (preg_match('/MAGW|MAGWJS/', $ua, $matches)) {
        $infos = array_merge($infos, array(
            'platform_company'  =>'Gateway (PC)',
        ));
    }

    /* Check Fujitsu OEM computers */
    elseif (preg_match('/MAFS|MAFSJS|FSJB/', $ua, $matches)) {
        $infos = array_merge($infos, array(
            'platform_company'  =>'Fujitsu (PC)',
        ));
    }

    /* Check Hyrican OEM computers */
    elseif (preg_match('/MANM|MANMJS/', $ua, $matches)) {
        $infos = array_merge($infos, array(
            'platform_company'  =>'Hyrican (PC)',
        ));
    }

    /* Check Compaq OEM computers */
    elseif (preg_match('/CPDTDF|CPNTDF|CMNTDF|CMNTDFJS|CMDTDF|CMDTDFJS/', $ua, $matches)) {
        $infos = array_merge($infos, array(
            'platform_company'  =>'Compaq (PC)',
        ));
    }

    /* Check Polaroid devices, to be inserted before Xiaomi */
    elseif (preg_match('/ (MID[0-9A-Z]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Polaroid.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_PolaroidClientDevice($model));
    } elseif (preg_match('/; (PRO6084PGE01)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Polaroid.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_PolaroidClientDevice($model));
    }

    /* Check Umi devices, to be inserted before Xiaomi */
    elseif (preg_match('/ UMI([_ \-][0-9A-Za-z_ \-]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Umi.php.inc');
        $model = trim(preg_replace('/_|-/', ' ', $matches[1]));
        $infos = array_merge($infos, get_UmiClientDevice($model));
    } elseif (preg_match('/; Android [0-9\.]+; (IRON|Rome|Hammer|eMax|Touch|Super)([ |_])?(Pro|X|S|Mini|4G)?( Build|\))[^$]/Ui', $ua, $matches)) {
        include_once('clientDevices/Umi.php.inc');
        $model = trim($matches[1].' '.$matches[3]);
        $infos = array_merge($infos, get_UmiClientDevice($model));
    }

    /* Check VKWorld devices, to be inserted before Xiaomi */
    elseif (preg_match('/ (VK-?[A-Z0-9\- ]+)( Build|\)|\()/i', $ua, $matches)) {
        include_once('clientDevices/VKWorld.php.inc');
        $infos = array_merge($infos, get_VKWorldClientDevice($matches[1]));
    } elseif (preg_match('/ (Mix Plus)( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/VKWorld.php.inc');
        $infos = array_merge($infos, get_VKWorldClientDevice($matches[1]));
    }

    /* Check FNF devices, to be inserted before Xiaomi */
    elseif (preg_match('/ (iFive[A-Za-z0-9 _]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/FNF.php.inc');
        $model = trim(preg_replace('/_/', ' ', $matches[1]));
        $infos = array_merge($infos, get_FNFClientDevice($model));
    }

    /* Check Pocophone devices, before Xiaomi */
    elseif (($device = _client_isPocophoneDevice($ua)) !== false) {
        $infos = array_merge($infos, get_PocophoneClientDevice($device));
    }

    /* Check Xiaomi devices */
    elseif (preg_match('/; (MI)( |-)?([A-Za-z0-9][A-Za-z0-9 ]{0,})( Build|\)| MIUI)/Ui', $ua, $matches)) {
        include_once('clientDevices/Xiaomi.php.inc');
        $infos = array_merge($infos, get_XiaomiClientDevice($matches[1].' '.$matches[3]));
    } elseif (preg_match('/; (Redmi )(.*)( Build|\)| MIUI)/Ui', $ua, $matches)) {
        include_once('clientDevices/Xiaomi.php.inc');
        $infos = array_merge($infos, get_XiaomiClientDevice($matches[1].$matches[2]));
    } elseif (preg_match('/; (M(19|2[01])([01][0-9][CJ]|[01][123]K)[0-9]{1,2}[A-Z]{0,3})( Build|\)| MIUI)/Ui', $ua, $matches)) {
        include_once('clientDevices/Xiaomi.php.inc');
        $infos = array_merge($infos, get_XiaomiClientDevice($matches[1]));
    } elseif (preg_match('/; (SHARK )?([A-Z]{3})(|-)?(H0)( Build|\)| MIUI)/Ui', $ua, $matches)) {
        include_once('clientDevices/Xiaomi.php.inc');
        $infos = array_merge($infos, get_XiaomiClientDevice($matches[2].' '.$matches[4]));
    } elseif (preg_match('/; (HM NOTE )(.*)( Build|\)| MIUI)/Ui', $ua, $matches)) {
        include_once('clientDevices/Xiaomi.php.inc');
        $infos = array_merge($infos, get_XiaomiClientDevice($matches[1].$matches[2]));
    } elseif (preg_match('/; (2014811|dandelion)( Build|\)| MIUI)/Ui', $ua, $matches)) {
        include_once('clientDevices/Xiaomi.php.inc');
        $infos = array_merge($infos, get_XiaomiClientDevice($matches[1]));
    }

    /* Check Iris Sat devices */
    elseif (preg_match('/(I-S[0-9])/', $ua, $matches)) {
        $rep = array();
        $model = trim($matches[1]);
        $infos = array_merge($infos, array(
            'platform_company'  => 'Iris Sat',
            'platform_model'    => $model,
            'platform_type'     => 'handheld',
            'platform_format'   => 'handheld',
            ),
            isset($rep[$model])?$rep[$model]:array()
        );
    }
    elseif (preg_match('/(VOX POP plus)/', $ua, $matches)) {
        $rep = array(
            'VOX POP plus' => array('platform_model' => 'Vox Pop Plus',),
        );
        $model = trim($matches[1]);
        $infos = array_merge($infos, array(
            'platform_company'  => 'Iris Sat',
            'platform_model'    => $model,
            'platform_type'     => 'handheld',
            'platform_format'   => 'handheld',
            ),
            isset($rep[$model])?$rep[$model]:array()
        );
    }

    /* Check Kazam devices */
    elseif (preg_match('/KAZAM_(Trooper_450L)/', $ua, $matches)) {
        include_once('clientDevices/Kazam.php.inc');
        $model = trim($matches[1]);
        $infos = array_merge($infos, get_KazamClientDevice($model));
    }

    /* Check Condor devices */
    elseif (preg_match('/(PGN|PHQ|SP|PAM|PFS|TCV|TGW)\-?([0-9]{3}G?)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Condor.php.inc');
        $infos = array_merge($infos, get_CondorClientDevice($matches[1].$matches[2]));
    } elseif (preg_match('/(Griffe|Allure|Plume) ([A-Za-z0-9 ]{1,})( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Condor.php.inc');
        $infos = array_merge($infos, get_CondorClientDevice($matches[1].' '.$matches[2]));
    }

    /* Check Smartisan devices */
    elseif (preg_match('/ (OC|OD|OS|SM|YQ)([0-9]{3})( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Smartisan.php.inc');
        $model = $matches[1].$matches[2];
        $infos = array_merge($infos, get_SmartisanClientDevice($model));
    }

    /* Check Gigabyte devices */
    elseif (preg_match('/(GSmart Rey R3)/', $ua, $matches)) {
        include_once('clientDevices/Gigabyte.php.inc');
        $model = trim($matches[1]);
        $infos = array_merge($infos, get_GigabyteClientDevice($model));
    }

    /* Check Lyf devices */
    elseif (preg_match('/(LYF\/F[0-9]{2,3}[A-Z])\//', $ua, $matches)) {
        include_once('clientDevices/Lyf.php.inc');
        $infos = array_merge($infos, get_LyfClientDevice($matches[1]));
    }

    /* Check Infinix devices */
    elseif (preg_match('/; Infinix[ |-]([^\)]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Infinix.php.inc');
        $infos = array_merge($infos, get_InfinixClientDevice($matches[1]));
    }

    /* Check Digital Alpha servers */
    elseif (preg_match('/ (Digital AlphaServer [A-Za-z0-9]*) /', $ua, $matches)) {
        $model = trim($matches[1]);
        $infos = array_merge($infos, array(
            'platform_company'  => 'Digital Equipment Corporation (PC)',
            'platform_model'    => $model,
        ));
    }

    /* Check Vodafone devices */
    elseif (preg_match('/ Vodafone ([A-Za-z0-9]{3}N)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Vodafone.php.inc');
        $model = trim($matches[1]);
        $infos = array_merge($infos, get_VodafoneClientDevice($model));
    } elseif (preg_match('/; (VF\-?[0-9]{3,4})( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Vodafone.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_VodafoneClientDevice($model));
    }

    /* Check MobiWire devices */
    elseif (preg_match('/ (Taima) Build/', $ua, $matches)) {
        include_once('clientDevices/MobiWire.php.inc');
        $model = trim($matches[1]);
        $infos = array_merge($infos, get_MobiWireClientDevice($model));
    }

    /* Check Oppo devices */
    elseif (preg_match('/ (1201|A37f|A37fw|A1601|F1f|PA[FD]M00|PDEM30|CPH[0-9]{4})( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Oppo.php.inc');
        $infos = array_merge($infos, get_OppoClientDevice($matches[1]));
    } elseif (preg_match('/OPPO ([^.]+) Build/', $ua, $matches)) {
        include_once('clientDevices/Oppo.php.inc');
        $infos = array_merge($infos, get_OppoClientDevice($matches[1]));
    }

    /* Check Realme devices */
    elseif (preg_match('/ (RMX[0-9]{4})( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Realme.php.inc');
        $infos = array_merge($infos, get_RealmeClientDevice($matches[1]));
    }

    /* Check Yotaphone devices */
    elseif (preg_match('/(YD20[0-9]) Build/', $ua, $matches)) {
        include_once('clientDevices/Yotaphone.php.inc');
        $infos = array_merge($infos, get_YotaphoneClientDevice($matches[1]));
    } elseif (preg_match('/ YotaPhone ([0-9]) Build/', $ua, $matches)) {
        include_once('clientDevices/Yotaphone.php.inc');
        $infos = array_merge($infos, get_YotaphoneClientDevice($matches[1]));
    }

    /* Check eBest devices */
    elseif (preg_match('/EBEST ([^.]*) Build/', $ua, $matches)) {
        $model = trim($matches[1]);
        $infos = array_merge($infos, array(
            'platform_company'  => 'eBest',
            'platform_model'    => $model,
            'platform_type'     => 'handheld',
            'platform_format'   => 'handheld',
        ));
    }

    /* Check Vivo devices */
    elseif (preg_match('/; ([0-9]{4})( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Vivo.php.inc');
        $infos = array_merge($infos, get_VivoClientDevice($matches[1]));
    } elseif (preg_match('/; vivo (.*)( Build|\)|;)/Ui', $ua, $matches)) {
        include_once('clientDevices/Vivo.php.inc');
        $infos = array_merge($infos, get_VivoClientDevice($matches[1]));
    } elseif (preg_match('/; (V(19|20|21)(2[0-9]|30|49)[A|T]?)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Vivo.php.inc');
        $infos = array_merge($infos, get_VivoClientDevice(strtoupper($matches[1])));
    }

    /* Check Sharp devices */
    elseif (preg_match('/(SH\-[0-9]{2}[A-Z]|SH\-[A-Z][0-9]{2}|SH[A-Z][0-9]{2}|FS[0-9]{4}|S3\-SH)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Sharp.php.inc');
        $infos = array_merge($infos, get_SharpClientDevice($matches[1]));
    }

    /* Check Crosscall devices */
    elseif (preg_match('/(Action|Trekker|Odyssey|Core)(\-|_| )([0-9\ A-Z]+)( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/Crosscall.php.inc');
        $infos = array_merge($infos, get_CrosscallClientDevice(strtolower($matches[1].'-'.$matches[3])));
    }

    /* Check SPC devices, before ZTE */
    elseif (preg_match('/; (Z2)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/SPC.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_SPCClientDevice($device));
    }

    /* Check ZTE devices : before acer */
    elseif (preg_match('/; ?ZTE ([a-zA-Z0-9 ]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/ZTE.php.inc');
        $infos = array_merge($infos, get_ZTEClientDevice($matches[1]));
    } elseif (preg_match('/; (Z[0-9]{1,3}[A-Z]{0,2})( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/ZTE.php.inc');
        $infos = array_merge($infos, get_ZTEClientDevice($matches[1]));
    } elseif (preg_match('/ (NX595J)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/ZTE.php.inc');
        $infos = array_merge($infos, get_ZTEClientDevice($matches[1]));
    } elseif (preg_match('/ (OPENC|);/i', $ua, $matches)) {
        include_once('clientDevices/ZTE.php.inc');
        $infos = array_merge($infos, get_ZTEClientDevice($matches[1]));
    }

    /* Check Ampe devices */
    elseif (($device = _client_isAmpeDevice($ua)) !== false) {
        $infos = array_merge($infos, get_AmpeClientDevice($device));
    }

    /* Check IMO devices */
    elseif (preg_match('/ IMO_([a-zA-Z0-9]*) Build/', $ua, $matches)) {
        include_once('clientDevices/IMO.php.inc');
        $infos = array_merge($infos, get_IMOClientDevice($matches[1]));
    }

    /* Check Lemon devices */
    elseif (preg_match('/ Lemon ([a-zA-Z0-9\- ]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Lemon.php.inc');
        $infos = array_merge($infos, get_LemonClientDevice($matches[1]));
    }

    /* Check Evertek devices */
    elseif (preg_match('/ (Ever[a-zA-Z0-9\- ]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Evertek.php.inc');
        $infos = array_merge($infos, get_EvertekClientDevice($matches[1]));
    }

    /* Check Essential devices */
    elseif (preg_match('/ (PH-1)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Essential.php.inc');
        $infos = array_merge($infos, get_EssentialClientDevice($matches[1]));
    }

    /* Check Digicel devices */
    elseif (preg_match('/(DL[0-9A-Za-z ]+)( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/Digicel.php.inc');
        $model = trim($matches[1]);
        $infos = array_merge($infos, get_DigicelClientDevice($model));
    }

    /* Check Meizu devices */
    elseif (preg_match('/ (MZ-[A-Za-z0-9_ ]*)( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/Meizu.php.inc');
        $model = trim($matches[1]);
        $infos = array_merge($infos, get_MeizuClientDevice($model));
    } elseif (preg_match('/ Meizu[ |_]([A-Za-z0-9_ ]*)( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/Meizu.php.inc');
        $model = trim($matches[1]);
        $infos = array_merge($infos, get_MeizuClientDevice($model));
    } elseif (preg_match('/ (MX[0-9])( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/Meizu.php.inc');
        $model = trim($matches[1]);
        $infos = array_merge($infos, get_MeizuClientDevice($model));
    } elseif (preg_match('/ (M3s|Note9|m2 note|16th|U20)( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Meizu.php.inc');
        $model = trim($matches[1]);
        $infos = array_merge($infos, get_MeizuClientDevice($model));
    } elseif (preg_match('/ (PRO [0-9][.*]*)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Meizu.php.inc');
        $model = trim($matches[1]);
        $infos = array_merge($infos, get_MeizuClientDevice($model));
    }

    /* Check Symphony devices */
    elseif (preg_match('/ ([iEHG][0-9]{2,3}) Build/i', $ua, $matches)) {
        include_once('clientDevices/Symphony.php.inc');
        $model = trim($matches[1]);
        $infos = array_merge($infos, get_SymphonyClientDevice($model));
    }

    /* Check Gionee devices */
    elseif (preg_match('/ (M[2-3|5-7])(_[Lite|Plus|Mini|Mirror])?( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Gionee.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_GioneeClientDevice($model));
    } elseif (preg_match('/ (P8W|F103 Pro)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Gionee.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_GioneeClientDevice($model));
    }

    /* Check Konrow devices */
    elseif (preg_match('/ (BIGCOOL|Just5|EASY5)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Konrow.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_KonrowClientDevice($model));
    }

    /* Check HomTom devices : before Umidigi */
    elseif (preg_match('/; (HT[0-9]{1,2})([A-Z]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/HomTom.php.inc');
        $model = $matches[1].trim($matches[2]);
        $infos = array_merge($infos, get_HomTomClientDevice($model));
    } elseif (preg_match('/; (Z11)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/HomTom.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_HomTomClientDevice($model));
    }

    /* Check Vernee devices */
    elseif (preg_match('/ (Apollo|Thor|Mars|V2 )([A-Za-z0-9]*)?( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/Vernee.php.inc');
        $model = $matches[1].$matches[2];
        $infos = array_merge($infos, get_VerneeClientDevice($model));
    } elseif (preg_match('/ vernee_([[A-Za-z0-9]*)( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/Vernee.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_VerneeClientDevice($model));
    }

    /* Check Ken Xin Da devices */
    elseif (preg_match('/ (Three_Proofings_W9)/i', $ua, $matches)) {
        include_once('clientDevices/KenXinDa.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_KenXinDaClientDevice($model));
    }

    /* Check Qilive devices */
    elseif (preg_match('/ (Q[0-9]+S[A-Za-z0-9]+)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Qilive.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_QiliveClientDevice($model));
    }

    /* Check Quantum devices */
    elseif (preg_match('/ Quantum ([^\)]*)( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/Quantum.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_QuantumClientDevice($model));
    }

    /* Check Verykool devices */
    elseif (preg_match('/ (LUNA_II)( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/Verykool.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_VerykoolClientDevice($model));
    }

    /* Check Ulefone devices */
    elseif (preg_match('/ (s10_pro|S11|Note 8P|Armor[ |_].*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Ulefone.php.inc');
        $device = strtolower($matches[1]);
        $infos = array_merge($infos, get_UlefoneClientDevice($device));
    } elseif (preg_match('/; Ulefone_(.*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Ulefone.php.inc');
        $device = strtolower($matches[1]);
        $infos = array_merge($infos, get_UlefoneClientDevice($device));
    }

    /* Check Neffos devices */
    elseif (preg_match('/ Neffos[ |_]([^\)]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Neffos.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_NeffosClientDevice($device));
    }

    /* Check T-Mobile devices */
    elseif (preg_match('/ T-Mobile[ |_]([^\)]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/T-Mobile.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_TMobileClientDevice($device));
    } elseif (preg_match('/; (REVVL 2)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/T-Mobile.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_TMobileClientDevice($device));
    }

    /* Check i.safe devices */
    elseif (preg_match('/ (IS[0-9_]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Isafe.php.inc');
        $device = trim($matches[1]);
        $infos = array_merge($infos, get_IsafeClientDevice($device));
    }

    /* Check It Works devices */
    elseif (preg_match('/ (TM[0-9]{3,4})([A-Z]{3})?( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/ItWorks.php.inc');
        $device = trim($matches[1].$matches[2]);
        $infos = array_merge($infos, get_ItWorksClientDevice($device));
    }

    /* Check Torque devices */
    elseif (preg_match('/ (Edge)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Torque.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_TorqueClientDevice($model));
    }

    /* Check Wieppo devices */
    elseif (preg_match('/ Wieppo ([^\)]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Wieppo.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_WieppoClientDevice($model));
    }

    /* Check Mecool devices */
    elseif (preg_match('/; (M8S PRO L)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Mecool.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_MecoolClientDevice($model));
    }

    /* Check Cube devices */
    elseif (preg_match('/; (X1)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Cube.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_CubeClientDevice($model));
    }

    /* Check Xtreamer mobiles */
    elseif (preg_match('/; (Xtreamer )?(Bang|Joyz|Aiki\-[0-9]S?)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Xtreamer.php.inc');
        $model = $matches[2];
        $infos = array_merge($infos, get_XtreamerClientDevice($model));
    /* Check Xtreamer tablets */
    } elseif (preg_match('/; (X[0-9]\-[A-Z0-9]{2,4})( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Xtreamer.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_XtreamerClientDevice($model));
    }

    /* Check Wileyfox devices */
    elseif (preg_match('/; Wileyfox (.*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Wileyfox.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_WileyfoxClientDevice($model));
    }

    /* Check American Network Solutions devices */
    elseif (preg_match('/; (UL40)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/AmericanNetworkSolutions.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_AmericanNetworkSolutionsClientDevice($model));
    }

    /* Check Intex devices */
    elseif (preg_match('/; (Aqua 5.5 VR)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Intex.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_IntexClientDevice($device));
    }

    /* Check Krüger & Matz devices */
    elseif (preg_match('/ (FLOW)( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/KrugerMatz.php.inc');
        $device = strtolower($matches[1]);
        $infos = array_merge($infos, get_KrugerMatzClientDevice($device));
    }

    /* Check Hyundai devices */
    elseif (preg_match('/; (G25524K)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Hyundai.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_HyundaiClientDevice($device));
    }

    /* Check Dexp devices */
    elseif (preg_match('/; (Ixion [^.]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Dexp.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_DexpClientDevice($device));
    }

    /* Check Vertex devices */
    elseif (preg_match('/; (Impress_[^.]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Vertex.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_VertexClientDevice($device));
    }

    /* Check Danew devices */
    elseif (preg_match('/; (Dslide1013QC_v2|Konnect_[0-9]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Danew.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_DanewClientDevice($device));
    }

    /* Check Chuwi devices */
    elseif (preg_match('/; (Hi[0-9]+ ?(pro|Air))( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Chuwi.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_ChuwiClientDevice($device));
    }

    /* Check Xtouch devices */
    elseif (preg_match('/; Xtouch ([^.]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Xtouch.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_XtouchClientDevice($device));
    }

    /* Check MPMan devices */
    elseif (preg_match('/; (MP[^.]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/MPMan.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_MPManClientDevice($device));
    }

    /* Check Timmy devices */
    elseif (preg_match('/; (M12)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Timmy.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_TimmyClientDevice($device));
    }

    /* Check Öwn devices */
    elseif (preg_match('/; (FUN_VALUE)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Own.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_OwnClientDevice($device));
    }

    /* Check Nvidia devices */
    elseif (preg_match('/; (SHIELD .*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Nvidia.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_NvidiaClientDevice($device));
    }

    /* Check Fairphone devices */
    elseif (preg_match('/; (FP[0-9]U?)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Fairphone.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_FairphoneClientDevice($device));
    }

    /* Check Echo devices */
    elseif (preg_match('/; (Stellar 4G)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Echo.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_EchoClientDevice($device));
    }

    /* Check LeEco devices */
    elseif (preg_match('/; (LE ?X[0-9]+)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/LeEco.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_LeEcoClientDevice($device));
    }

    /* Check Orbic devices */
    elseif (preg_match('/; (RC5[0-9]{2}L)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Orbic.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_OrbicClientDevice($device));
    }

    /* Check Palm devices */
    elseif (preg_match('/; (PVG100)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Palm.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_PalmClientDevice($device));
    }

    /* Check Poptel devices */
    elseif (preg_match('/; (P10|P9000_MAX)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Poptel.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_PoptelClientDevice($device));
    }

    /* Check Philips devices */
    elseif (preg_match('/\(Philips\, (PFS5803)\,/Ui', $ua, $matches)) {
        include_once('clientDevices/Philips.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_PhilipsClientDevice($device));
    }

    /* Check Artizlee devices */
    elseif (preg_match('/; (ATL-[0-9]{2}[A-Z]?)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Artizlee.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_ArtizleeClientDevice($device));
    }

    /* Check GamePad Digital devices */
    elseif (preg_match('/; (XD)( Plus)?( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/GamePadDigital.php.inc');
        $device = $matches[1].$matches[2];
        $infos = array_merge($infos, get_GamePadDigitalClientDevice($device));
    }

    /* Check True devices */
    elseif (preg_match('/; (SMART TAB 4G Speedy .*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/True.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_TrueClientDevice($device));
    }

    /* Check Cherry devices */
    elseif (preg_match('/; (Flare.*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Cherry.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_CherryClientDevice($device));
    }

    /* Check Sonim devices */
    elseif (preg_match('/; (XP[0-9]{4})( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Sonim.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_SonimClientDevice($device));
    }

    /* Check Xgody devices */
    elseif (preg_match('/; (T702)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Xgody.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_XgodyClientDevice($device));
    }

    /* Check Energy Sistem devices */
    elseif (preg_match('/; Energy (Phone|Tablet) (.*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/EnergySistem.php.inc');
        $device = $matches[1].' '.$matches[2];
        $infos = array_merge($infos, get_EnergySistemClientDevice($device));
    }

    /* Check Lumitel devices */
    elseif (preg_match('/; (L950[2-3])( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Lumitel.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_LumitelClientDevice($device));
    }

    /* Check Time2 devices */
    elseif (preg_match('/; (GC750C)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Time2.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_Time2ClientDevice($device));
    }

    /* Check Meiigoo devices */
    elseif (preg_match('/; (M1)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Meiigoo.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_MeiigooClientDevice($device));
    }

    /* Check Occulus devices */
    elseif (preg_match('/; (Quest)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Occulus.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_OcculusClientDevice($device));
    }

    /* Check Essentiel B devices */
    elseif (preg_match('/; (HEYOU[0-9]+|Smart_TAB_1003s|SmartTab1005)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/EssentielB.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_EssentielBClientDevice($device));
    }

    /* Check AllView devices */
    elseif (preg_match('/; (A10_Lite_2GB)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/AllView.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_AllViewClientDevice($device));
    }

    /* Check Strong devices */
    elseif (preg_match('/; (SN6B6BL_EU012)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Strong.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_StrongClientDevice($device));
    }

    /* Check Nubia devices */
    elseif (preg_match('/; (NX591J)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Nubia.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_NubiaClientDevice($device));
    }

    /* Check Kobo devices */
    elseif (preg_match('/\(Kobo ([^ ]*)/', $ua, $matches)) {
        include_once('clientDevices/Kobo.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_KoboClientDevice($device));
    }

    /* Check Onix devices */
    elseif (preg_match('/; ONIX (.*)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Onix.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_OnixClientDevice($device));
    }

    /* Check RCA devices */
    elseif (preg_match('/; (RCT6[0-9]{3}W[0-9]{2})( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/RCA.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_RCAClientDevice($device));
    }

    /* Check Teclast devices */
    elseif (preg_match('/; (TLA002|(M40|P20HD)_EEA)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Teclast.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_TeclastClientDevice($device));
    }

    /* Check Razer devices : before Blackview */
    elseif (preg_match('/; (Phone 2)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Razer.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_RazerClientDevice($device));
    }

    /* Check Acer devices : before Blackview, Wiko, Umidigi */
    elseif (preg_match('/ ([EISVZ][0-9]{3}|[EST][0-9]{2}|[AB][0-9]-[A0-9][0-9]0[A-Z]*)( Build|\))/', $ua, $matches)) {
        include_once('clientDevices/Acer.php.inc');
        $infos = array_merge($infos, get_AcerClientDevice($matches[1]));
    } elseif (preg_match('/ acer[_| |-]([A-Z][0-9]{3})( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Acer.php.inc');
        $infos = array_merge($infos, get_AcerClientDevice($matches[1]));
    } elseif (preg_match('/; Acer (.*)( Build| \(|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Acer.php.inc');
        $infos = array_merge($infos, get_AcerClientDevice($matches[1]));
    } elseif (preg_match('/ (Z2_WVGA_QXH|Z6PLUS|Liquid|A210)/', $ua, $matches)) {
        include_once('clientDevices/Acer.php.inc');
        $infos = array_merge($infos, get_AcerClientDevice($matches[1]));
    }

    /* Check Blackview devices : before Umidigi */
    elseif (preg_match('/; (BV|P)([0-9]{2,3}00[a-zA-Z0-9\-_ ]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Blackview.php.inc');
        $device = preg_replace('/-|_/', ' ', trim($matches[1].$matches[2]));
        $infos = array_merge($infos, get_BlackviewClientDevice($device));
    } elseif (preg_match('/; (S8|A[68]0?)(Pro| Max)?( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Blackview.php.inc');
        $device = trim($matches[1].$matches[2]);
        $infos = array_merge($infos, get_BlackviewClientDevice($device));
    }

    /* Check Umidigi devices */
    elseif (preg_match('/; (Umidigi )?(C Note[ ]{0,1}[0-9]*|[A|S|Z][0-9]*)[ |\-|_]?(Pro|Lite)?( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Umidigi.php.inc');
        $model = ucwords(strtolower($matches[2].($matches[3] != '' ? ' '.$matches[3] : '')));
        $infos = array_merge($infos, get_UmidigiClientDevice($model));
    } elseif (preg_match('/; Umidigi (X)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Umidigi.php.inc');
        $model = ucwords(strtolower($matches[1]));
        $infos = array_merge($infos, get_UmidigiClientDevice($model));
    } elseif (preg_match('/; (Power|BISON)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Umidigi.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_UmidigiClientDevice($model));
    }

    /* Check Gigaset devices */
    elseif (preg_match('/; Gigaset (.*)?( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Gigaset.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_GigasetClientDevice($model));
    }

    /* Check Teeno devices */
    elseif (preg_match('/; (Vmobile .*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Teeno.php.inc');
        $model = $matches[1];
        $infos = array_merge($infos, get_TeenoClientDevice($model));
    }

    /* Check Wiko */
    elseif (preg_match('/; (barry|birdy|bloom|cink|darkfull|darkmoon|darkside|fever|fizz|freddy|getaway|goa|harry|highway|iggy|jerry|jimmy|kenny|kite|lenny|ozzy|pulp|rainbow|ridge|robby|selfy|slide|stairway|star|sublim|sunny|sunset|tommy|u feel|u pulse|view|violet|wax|wim)([A-Za-z0-9 _]*)?( Build|\))/i', $ua, $matches)) {
        $model = trim(strtolower($matches[1].(isset($matches[2])?$matches[2]:'')));
        include_once('clientDevices/Wiko.php.inc');
        $infos = array_merge($infos, get_WikoClientDevice($model));
    } elseif (preg_match('/ (W[_\-][A-Z][0-9]{3}[A-Z]?)(\-[A-Z]{3})?( Build|\))/i', $ua, $matches)) {
        $model = trim(strtolower($matches[1]));
        include_once('clientDevices/Wiko.php.inc');
        $infos = array_merge($infos, get_WikoClientDevice($model));
    } elseif (preg_match('/ ([S|V][0-9]{2}[A-Za-z0-9\(\)]*)( Build|\))/i', $ua, $matches)) {
        $model = trim(strtolower($matches[1]));
        include_once('clientDevices/Wiko.php.inc');
        $infos = array_merge($infos, get_WikoClientDevice($model));
    }

    /* Check Quanta, before Logicom */
    elseif (preg_match('/(Le Pan TC802A)( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/Quanta.php.inc');
        $infos = array_merge($infos, get_QuantaClientDevice($matches[1]));
    }

    /* Check Logicom, after Wiko */
    elseif (preg_match('/L-EMENT_?([^.]*)( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/Logicom.php.inc');
        $infos = array_merge($infos, get_LogicomClientDevice($matches[1]));
    } elseif (preg_match('/ (M bot Tab [0-9]{3})( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Logicom.php.inc');
        $infos = array_merge($infos, get_LogicomClientDevice($matches[1]));
    } elseif (preg_match('/; (L[a|e][_| ]?[^.]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Logicom.php.inc');
        $infos = array_merge($infos, get_LogicomClientDevice($matches[1]));
    } elseif (preg_match('/; Logicom (.*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Logicom.php.inc');
        $infos = array_merge($infos, get_LogicomClientDevice($matches[1]));
    }

    /* Check CellAllure devices */
    elseif (preg_match('/; (Fashion C)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/CellAllure.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_CellAllureClientDevice($device));
    }

    /* Check X-Tigi devices */
    elseif (preg_match('/; (JOY10 Mate)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/XTigi.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_XTigiClientDevice($device));
    } elseif (preg_match('/; X[_|\-]TIGI_(.*)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/XTigi.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_XTigiClientDevice($device));
    }

    /* Check Mobicel devices */
    elseif (preg_match('/; (ASTRO)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Mobicel.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_MobicelClientDevice($device));
    }

    /* Check Fujitsu devices */
    elseif (preg_match('/; (F-03F)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Fujitsu.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_FujitsuClientDevice($device));
    }

    /* Check Smartak devices */
    elseif (preg_match('/; (TAB910)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Smartak.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_SmartakClientDevice($device));
    }

    /* Check Edenwood devices */
    elseif (preg_match('/; (W55HD)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Edenwood.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_EdenwoodClientDevice($device));
    }

    /* Check Colors devices */
    elseif (preg_match('/; (X109)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Colors.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_ColorsClientDevice($device));
    }

    /* Check Docomo devices */
    elseif (preg_match('/; (d-[0-9]{2}[A-Z])( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Docomo.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_DocomoClientDevice($device));
    }

    /* Check Telma devices */
    elseif (preg_match('/; telma (.*)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Telma.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_TelmaClientDevice($device));
    }

    /* Check Verizon devices */
    elseif (preg_match('/; (QMV7[AB])( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Verizon.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_VerizonClientDevice($device));
    }

    /* Check Moqi devices */
    elseif (preg_match('/; (I7S)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Moqi.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_MoqiClientDevice($device));
    }

    /* Check Gome devices */
    elseif (preg_match('/; GOME_([A-Za-z0-9_]+)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Gome.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_GomeClientDevice($device));
    }

    /* Check Excelvan devices */
    elseif (preg_match('/; (QT\-10)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Excelvan.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_ExcelvanClientDevice($device));
    }

    /* Check Ace devices */
    elseif (preg_match('/; (BUZZ|URBAN|CLEVER) ([0-9])( LITE| PRO| PLUS)?( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Ace.php.inc');
        $device = $matches[1].' '.$matches[2].$matches[3];
        $infos = array_merge($infos, get_AceClientDevice($device));
    }

    /* Check Syh devices */
    elseif (preg_match('/; (Young Y56)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Syh.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_SyhClientDevice($device));
    }

    /* Check TCL devices */
    elseif (preg_match('/; (T810S|T773O|T799H)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/TCL.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_TCLClientDevice($device));
    }

    /* Check Mara Phones devices */
    elseif (preg_match('/; Mara Phones (.*)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/MaraPhones.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_MaraPhonesClientDevice($device));
    }

    /* Check Selecline devices */
    elseif (preg_match('/; (ST19101)( Build|\))/U', $ua, $matches)) {
        include_once('clientDevices/Selecline.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_SeleclineClientDevice($device));
    }

    /* Check known Unknown devices */
    elseif (preg_match('/; (Phh-Treble|Unspecified Device).*( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Unknown.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_UnknownClientDevice($device));
    } elseif (preg_match('/; (K)\)/Ui', $ua, $matches)) {
        include_once('clientDevices/Unknown.php.inc');
        $device = $matches[1];
        $infos = array_merge($infos, get_UnknownClientDevice($device));
    }

    /* Check Roku/DVP-9.10 */
    elseif (preg_match('/^Roku\/DVP-([0-9\.]{3,})/', $ua, $matches))
        $infos = array_merge($infos, array(
            'platform_company'  => 'Roku',
            'platform_model'    => '',
            'platform_type'     => 'tvset',
            'platform_format'   => 'desktop',
            'system_title'      => 'Roku Digital Video Player',
            'system_version'    => isset($matches[1]) ? $matches[1] : '',
        ));

    /*** check operating system ***/
    if (!isset($infos['system_title'])) {
        /* Check Windows Phone */
        if (preg_match('/Windows Phone ?([^;]*);/', $ua, $matches))
            $infos = array_merge(array(
                'platform_type'     => 'handheld',
                'platform_format'   => 'handheld',
                'system_title'      => 'Windows Phone',
                'system_version'    => isset($matches[1]) ? $matches[1] : '',
                ), $infos
            );

        /* Check gen os with notified version (before Android/Macintosh) */
        elseif (preg_match('/(SymbianOS|webOS|KAIOS|Bada|Haiku|Tizen|NetCast|_TV_MT[0-9]{4})(?:\/| )?(R?[0-9][a-z0-9\._]*)?/i', $ua, $matches)) {
            $rep = array(
                'SymbianOS' => array('platform_type' => 'handheld', 'platform_format' => 'handheld',),
                'KAIOS'     => array('system_title' => 'KaiOS', 'platform_type' => 'handheld', 'platform_format' => 'handheld'),
                'Bada'      => array('platform_company' => 'Samsung', 'platform_type' => 'handheld', 'platform_format' => 'handheld',),
                'Haiku'     => array('platform_company' => 'PC', 'platform_type' => 'computer', 'platform_format' => 'desktop'),
                'Tizen'     => array('platform_company' => 'Samsung', 'platform_type' => 'tvset', 'platform_format' => 'desktop'),
                'NetCast'   => array('platform_company' => '',  'platform_type' => 'tvset', 'platform_format'   => 'desktop',),
                'Saphi'     => array('platform_company' => 'Philips', 'system_title' => 'Saphi', 'platform_type' => 'tvset', 'platform_format'   => 'desktop',),
            );
            if (preg_match('/^_TV_MT/i', $matches[1], $matches2))
                $system = 'Saphi';
            else
                $system = $matches[1];

            if (isset($matches[2]))
                $version = $matches[2];
            elseif (strtolower($system) == 'netcast' and preg_match('/ SmartTV\/([0-9\.]*)?/i', $ua, $matches2))
                $version = $matches2[1];
            else
                $version = '';
            $infos = array_merge($infos, array(
                    'system_title'      => $system,
                    'system_version'    =>  $version,
                ),
                isset($rep[$system]) ? $rep[$system] : array()
            );
        }

        /* Check Android */
        elseif (preg_match('/(Android|Adr)[; \/]*([0-9\.]*)?/', $ua, $matches)) {
            /* Check version */
            $version = isset($matches[2]) ? $matches[2] : '';
            $infos = array_merge(array(
                'platform_type'     => (preg_match('/tablet/i',$ua) ? 'tablet' : 'handheld'),
                'platform_format'   => (preg_match('/tablet/i',$ua) ? 'tablet' : 'handheld'),
                'system_title'      => 'Android',
                'system_version'    => $version,
                ),
                $infos
            );
        }

        /* Check Macintosh */
        elseif (preg_match('/(?:Intel |PPC |)Mac OS X( Version)?(Mach-O| [0-9\._]*|)/i', $ua, $matches)) {
            /* Check version */
            $version = ($matches[2] != ' Mach-O') ? substr(str_replace('_','.',$matches[2]), 1) : '';

            /* build info */
            $infos = array_merge($infos, array(
                'platform_company'  => 'Apple',
                'platform_model'    => 'Macintosh',
                'platform_type'     => 'computer',
                'platform_format'   => 'desktop',
                'system_title'      => version_compare ($version, '10.12') >= 0 ? 'macOS' : 'Mac OS X',
                'system_version'    => $version,
            ));
        } elseif (preg_match('/ Darwin\/([0-9\.]*) \(x86_64\)/i', $ua, $matches)) {
            /* Check version */
            $version = $matches[1];
            $version = explode('.', $version);
            $version = '10.'.((int)$version[0]-4); //.'.'.$version[1];

            /* build info */
            $infos = array_merge($infos, array(
                'platform_company'  => 'Apple',
                'platform_model'    => 'Macintosh',
                'platform_type'     => 'computer',
                'platform_format'   => 'desktop',
                'system_title'      => version_compare ($version, '10.12') >= 0 ? 'macOS' : 'Mac OS X',
                'system_version'    => $version
            ));
        }

        /* Check Classic Macintosh */
        elseif (preg_match('/Mac_PowerPC|Macintosh/i', $ua, $matches)) {
            /* build info */
            $infos = array_merge($infos, array(
                'platform_company'  => 'Apple',
                'platform_model'    => 'Macintosh',
                'platform_type'     => 'computer',
                'platform_format'   => 'desktop',
                'system_title'      => 'Mac OS Classic',
                'system_version'    => '',
            ));
        }

        /* Check determined linux os */
        elseif (preg_match('/(Debian|Ubuntu|DragonFly|SUSE|Slackware|CentOS|Fedora|Kubuntu|Edubuntu|Xubuntu|Gentoo|Red Hat|Mint|Mandriva|CrOS|Arch ?Linux|Caldera Systems OpenLinux|Sailfish|Raspbian|Mageia|Ordissimo)\W/i', $ua, $matches)) {
            $rep = array(
                'CrOS'      => array('system_title'     => 'Chrome OS (Linux)',),
                'Sailfish'  => array('platform_company' => '',  'system_title' => 'Sailfish OS (Linux)', 'platform_type' => 'handheld', 'platform_format' => 'handheld',),
            );
            $infos = array_merge(array(
                'platform_company'  => 'PC',
                'platform_model'    => '',
                'platform_type'     => 'computer',
                'platform_format'   => 'desktop',
                'system_title'      => ucwords($matches[1]).' (Linux)',
                'system_version'    => '',
                ),
                isset($rep[$matches[1]]) ? $rep[$matches[1]] : array()
            );
        }

        /* Check gen os with notified version */
        elseif (preg_match('/(AmigaOS|WebTV)(?:\/| )([0-9][a-z0-9\._]*)?/i', $ua, $matches))
            $infos = array_merge(array(
                'platform_company'  => 'PC',
                'platform_model'    => '',
                'platform_type'     => 'computer',
                'platform_format'   => 'desktop',
                'system_title'      => $matches[1],
                'system_version'    => isset($matches[2]) ? $matches[2] : ''
            ), $infos);

        /* Check gen os without notified version */
        elseif (preg_match('/(Linux|FreeBSD|SunOS|OpenBSD|SymbOS|OS\/2|BeOS|NetBSD|OpenSolaris|PalmSource|MAUI Runtime)[^\.]/', $ua, $matches)) {
            $rep = array(
                'SymbOS'        => array('platform_company' => '',          'system_title' => 'SymbianOS',      'platform_type' => 'handheld', 'platform_format'   => 'handheld',),
                'PalmSource'    => array('platform_company' => 'Palm',      'system_title' => 'PalmOS',         'platform_type' => 'handheld', 'platform_format'   => 'handheld',),
                'MAUI Runtime'  => array('platform_company' => '',          'system_title' => 'Maui (MediaTek)','platform_type' => 'handheld', 'platform_format'   => 'handheld',),
            );

            $model = trim($matches[1]);

            $infos = array_merge($infos, array(
                'platform_company'  => 'PC',
                'platform_model'    => '',
                'platform_type'     => 'computer',
                'platform_format'   => 'desktop',
                'system_title'      => $matches[1],
                'system_version'    => '',
                ),
                isset($rep[$model]) ? $rep[$model] : array()
            );
        }

        /* Check X11 gen unix  */
        elseif (preg_match('/X11/', $ua, $matches)) {
            $infos = array_merge($infos, array(
                'platform_company'  => 'PC',
                'platform_model'    => '',
                'platform_type'     => 'computer',
                'platform_format'   => 'desktop',
                'system_title'      => 'UNIX',
                'system_version'    => ''
                ),
                isset($rep[$matches[0]]) ? $rep[$matches[0]] : array()
            );
        }

        /* Check Windows */
        else if (preg_match('/(?:Windows;)?[^.]*(?:Windows |Windows|Win |Win)([^;\)]+)/', $ua, $matches)) {
            // '/(?:Windows |Win)([^;]{1,});/'

            /* build known windows versions */
            $w = array('3.1'=>'3.1', '95'=>'95', '98'=>'98', '9x 4.90' => '98', '9x4.90' => '98', 'CE'=>'CE',
                'ME'=>'ME', 'NT'=>'NT', 'NT3.51'=> 'NT 3.51',  'NT 4.0'=>'NT 4.0', 'NT4.0'=>'NT 4.0', 'NT 5.0'=>'2000',
                '2000'=>'2000', 'NT 5.1'=>'XP', 'XP'=>'XP', 'NT 5.2'=>'XP Pro / Server 2003 / Home Server', 'NT 6.0'=>'Vista',
                'NT 6.1'=>'7', 'NT6.1'=>'7', 'NT 6.1'=>'7', 'NT 6.2'=>'8', 'NT 6.3'=>'8.1', 'NT 6.4'=>'10',
                'NT 10.0'=>'10', '10'=>'10',
            );

            /* Load version */
            $v = $matches[1];
            if (substr($v, 0, 1) == ' ') $v = trim(substr($v, 0, 7));
            $version = isset($w[$v])?$w[$v]:$v;

            $infos = array_merge(array(
                'platform_company'  => 'PC',
                'platform_model'    => '',
                'platform_type'     => 'computer',
                'platform_format'   => 'desktop',
                'system_title'      => 'Windows',
                'system_version'    => $version,
                ), $infos);
        }
    }

    /* Check system and version */
    if (isset($infos['system_title']) and _client_checkSoftwareIsFake('system', $infos, $ua))
        return $fake;

    /*** check browser ***/

    /* Check opera mini/news */
    if (preg_match('/Opera (Mini|News)\/([0-9]+\.[0-9]+)/', $ua, $matches))
        $infos = array_merge(array(
            'browser_title'     => 'Opera '.$matches[1],
            'browser_version'   => $matches[2],
            'platform_type'     => 'handheld',
            'platform_format'   => 'handheld',
        ), $infos);

    /* Check opera */
    elseif (preg_match('/Opera(?:\/| )?([a-z0-9\._]*)?/i', $ua, $matches)) {
        if (preg_match('/Version\/([0-9\.]*)/', $ua, $matches2)) {
             $infos = array_merge(array(
                'browser_title'     => 'Opera',
                'browser_version'   => $matches2[1],
            ), $infos);
        } else {
            $infos = array_merge(array(
                'browser_title'     => 'Opera',
                'browser_version'   => $matches[1],
            ), $infos);
        }
    }

    /* Check unsmartly written browser, case sensitive */
    elseif (preg_match('/ (SE 2\.X)(?:\/| )?([a-z0-9\._]*)?/', $ua, $matches)) {
        $rep = array('SE 2.X'=>'Sogou Explorer');
        $infos = array_merge($infos, array(
            'browser_title'     => isset($rep[$matches[1]]) ? $rep[$matches[1]] : $matches[1],
            'browser_version'   => isset($matches[2]) ? $matches[2] : '',
        ));
    }

    /* Check unsmartly written browser, non case sensitive */
    elseif (preg_match('/(Maxthon|Opera|webissimo|NintendoBrowser|Acoo ?Browser|Deepnet Explorer|Avant Browser|BrowseX|OffByOne|KKman|Maemo Browser|BIDUBrowser|UBrowser|Airmail|EmbeddedWB|DX-Browser|Links|Sputnik|Instagram|Microsoft Outlook|Chromeless|Lunascape|JuziBrowser|LinkedInApp|Brave|360SE|liloiosapp|KAKAOTALK|Spotify|Ghostery)(?:\/| |\-| \/ |\:)?([a-z0-9\._]*)?/i', $ua, $matches)) {
        $rep = array('BIDUBrowser'=>'Baidu Brower', 'AcooBrowser' => 'Acoo Browser', 'EmbeddedWB'=>'Delphi Embedded Web Browser', 'LinkedInApp'=>'LinkedIn App', '360SE'=>'360Safe', 'liloiosapp'=>'Lilo', 'KAKAOTALK'=>'KakaoTalk');
        $infos = array_merge($infos, array(
            'browser_title'     => isset($rep[$matches[1]]) ? $rep[$matches[1]] : $matches[1],
            'browser_version'   => isset($matches[2]) ? $matches[2] : '',
        ));
    }

    /* Check L3 Chrome/Safari/Firefox/Explorer compatible browsers, to be tested before usual browsers */
    elseif (preg_match('/(MicroMessenger|ArcticFox|Yptp|InterWeb)(\/([a-z0-9\._]*))?/i', $ua, $matches)) {
        $rep = array('MicroMessenger' => 'WeChat',);
        $infos = array_merge($infos, array(
            'browser_title'     => isset($rep[$matches[1]]) ? $rep[$matches[1]] : $matches[1],
            'browser_version'   => isset($matches[3]) ? $matches[3] : '',
        ));
    }

    /* Check L2 Chrome/Safari/Firefox/Explorer compatible browsers, to be tested before usual browsers */
    elseif (preg_match('/(Yowser|MQQBrowser|Goanna|DingTalk|Konqueror|OculusBrowser|Lightning|Raindrop.io|Lilo)(\/([a-z0-9\._]*))?/i', $ua, $matches)) {
        $rep = array('MQQBrowser'=>'Mobile QQBrowser', 'OculusBrowser'=>'Oculus Browser');
        $infos = array_merge($infos, array(
            'browser_title'     => isset($rep[$matches[1]]) ? $rep[$matches[1]] : $matches[1],
            'browser_version'   => isset($matches[3]) ? $matches[3]:'',
        ));
    }

    /* Check Firefox compatible browser, using Firefox version */
    elseif (preg_match('/ Waterfox\) Gecko\/([0-9]{8}) Firefox\/([0-9a-zA-z\.]+)/', $ua, $matches)) {
        $infos = array_merge($infos, array(
            'browser_title'     => 'Waterfox',
            'browser_version'   => isset($matches[2]) ? $matches[2] : '',
        ));
    }

    /* Check Chrome compatible browser, using Chrome version */
    elseif (preg_match('/ Chrome\/([a-z0-9\.]*) (.*)(TwitterAndroid|LBBROWSER|Iron|DSBrowser|youcare-android-app|QQBrowser)/Ui', $ua, $matches)) {
        $rep = array('TwitterAndroid'=>'Twitter', 'LBBROWSER'=>'Liebao', 'youcare-android-app'=>'YouCare');
        $infos = array_merge($infos, array(
            'browser_title'     => isset($rep[$matches[3]]) ? $rep[$matches[3]] : $matches[3],
            'browser_version'   => $matches[1],
        ));
    }

    /* Check Chrome/Safari/Firefox/Explorer compatible browsers, with specific Version */
    elseif (preg_match('/ Version\/([a-z0-9\._]*)(.*) (Epiphany)/i', $ua, $matches)) {
        $rep = array();
        $infos = array_merge($infos, array(
            'browser_title'     => isset($rep[$matches[3]])?$rep[$matches[3]]:$matches[3],
            'browser_version'   => isset($matches[1])?$matches[1]:'',
        ));
    }

    /* Check Chrome/Safari/Firefox/Explorer compatible browsers, with specific details */
    elseif (preg_match('/(Viera|WebView|Ordissimo|TinyBrowser)\/([a-z0-9\._]*)?/i', $ua, $matches)) {
        $rep = array(
            'Viera'         => array('browser_title'=>'Viera', 'platform_type' => 'tvset'),
            'WebView'       => array('browser_title'=>'Android WebView', 'system_title'=>'Android', 'system_version'=>'', 'platform_company'=> '', 'platform_model' => '', 'platform_type' => 'handheld', 'platform_format' => 'handheld'),
            'Ordissimo'     => array('browser_title'=>'Navigateur Ordissimo', 'platform_company'=> 'Ordissimo'),
            'TinyBrowser'   => array('browser_title'=>'TinyBrowser', 'system_title'=>'Android', 'system_version'=>'', 'platform_company'=> '', 'platform_model' => '', 'platform_type' => 'handheld', 'platform_format' => 'handheld'),
        );
        $infos = array_merge($infos, array(
            'browser_title'     => isset($matches[1])?$matches[1]:'',
            'browser_version'   => isset($matches[2])?$matches[2]:'',
            ), (isset($rep[$matches[1]])?$rep[$matches[1]]:array())
        );
    }

    /* Check Facebook apps */
    elseif (preg_match('#(FBAN|FB_IAB)/([^;]*);([^.]*)(FBAV/[0-9\.]*)?;#i', $ua, $matches)) {
        $rep = array('FBIOS'=>'Facebook App', 'FB4A'=>'Facebook App', 'MessengerForiOS'=>'Facebook Messenger', 'Orca-Android'=>'Facebook Messenger', 'MessengerLiteForiOS' => 'Facebook Messenger');
        $infos = array_merge($infos, array(
            'browser_title'     => isset($rep[$matches[2]]) ? $rep[$matches[2]] : 'Unknown Facebook App ('.$matches[2].')',
            'browser_version'   => isset($matches[4]) ? substr($matches[4], 5) : '',
        ));
    }

    /* Check Chrome/Safari/Firefox/Explorer compatible browsers, to be tested before usual browsers */
    elseif (preg_match('/[ |\/](YaBrowser|Camino|OPR|IceDragon|Dragon|Comodo_Dragon|CriOS|Adventurer|Sleipnir|BoBrowser|Dolfin|ChromePlus|Phoenix|Shiretoko|NS8|MiuiBrowser|SamsungBrowser\/CrossApp|SamsungBrowser|Vivaldi|Qt|Iceweasel|Puffin|Firebird|FBAV|FBSV|app-ipad-CoteMaison|QQBrowser|PaleMoon|Arora|Edge|Cheshire|Getter|Dooble|amaya|Roccat|QupZilla|MMS|Iron|ACHEETAHI|OPiOS|Beonex|LibertyBrowser|LinkedIn|SeaMonkey|Cyberfox|Focus|Fennec|K-Meleon|CoolNovo|QuickLook|GSA|Electron|NG|FxiOS|FocusiOS|Lotus-Notes|Chimera|Epiphany|Iridium|HeadlessChrome|QwantiOS|QwantBrowser|QwantBrowser Firefox|QwantMobile|Blazer|TenFourFox|Silk|SailfishBrowser|coc_coc_browser|MZBrowser|2345Explorer|Kinza|WebPositive|Avast|VivoBrowser|Slimjet|Cornowser|baiduboxapp|Mint Browser|EdgA|Edg|EdgiOS|EdgW|AVG|Station|OPT|Waterfox|OppoBrowser|chromeframe|UR|QtWebEngine|handyCafeCln|Thunderbird|MxBrowser|mCent|PureBrowser|Valve Steam GameOverlay|Midori|HistoryHound|TO-Browser|K-Ninja|Kapiko|Kazehakase|eM Client|Mb2345Browser|LieBaoFast|Whale|ADG|Light|jp.co.yahoo.ipn.appli|HeyTapBrowser|HuaweiBrowser|OneNote|115Browser|Paparazzi!|rekonq|TaoBrowser|kioclient|DuckDuckGo|Xvast|Bunjalloo|Supermedium|SEMC-Browser|Surf|WebClip|Snapchat|Fluid|Ionic|1Password|PHX|AOL|Quark|Reeder|Polypane|AlohaBrowser|XWEB|RDDocuments|Minimo|Falkon|BingSapphire|BonEcho|GranParadiso|PCNBrowser|News Explorer)\/([a-z0-9\._]*)?/i', $ua, $matches)) {
        $rep = array('OPR'=>'Opera', 'CriOS'=>'Chrome', 'Comodo_Dragon'=>'Dragon', 'NS8'=>'Netscape', 'Firebird'=>'Thunderbird', 'FBAV'=>'Facebook App', 'FBSV'=>'Facebook App', 'app-ipad-CoteMaison'=>'C&ocirc;t&eacute; Maison App', 'amaya'=>'Amaya', 'MMS'=>'Opera Neon', 'ACHEETAHI'=>'CM Browser', 'OPiOS'=>'Opera mini', 'Focus'=>'Firefox Focus', 'GSA'=>'Google Search App', 'NG'=>'Browser NG', 'FxiOS'=>'Firefox', 'FocusiOS'=>'Firefox Focus', 'Lotus-Notes'=>'Lotus Notes', 'QwantiOS'=>'Qwant', 'QwantBrowser'=>'Qwant', 'QwantBrowser Firefox'=>'Qwant', 'QwantMobile' => 'Qwant', 'Blazer'=> 'Palm Blazer', 'SailfishBrowser' => 'Sailfish Browser', 'coc_coc_browser'=>'C&ocirc;c c&ocirc;c', 'MZBrowser' => 'Meizu Browser', 'Avast' => 'Avast Browser', 'VivoBrowser' => 'Vivo Browser', 'baiduboxapp' => 'Baidu Search App', 'EdgA' => 'Edge', 'Edg' => 'Edge', 'EdgiOS'=>'Edge', 'EdgW'=>'Edge', 'AVG'=>'AVG Secure Browser', 'OPT'=>'Opera Touch', 'chromeframe'=>'Internet Explorer/Chrome Frame', 'UR'=>'UR Browser', 'QtWebEngine'=>'QtWebEngine based browser', 'handyCafeCln'=>'HandyCafe', 'TO-Browser'=>'t-online.de Browser', 'K-Ninja'=>'K-Ninja Samourai', 'Mb2345Browser'=>'2345 Browser', 'ADG'=>'AOL Desktop', 'jp.co.yahoo.ipn.appli'=>'Yahoo Japan App', 'HeyTapBrowser'=>'HeyTap Browser', 'HuaweiBrowser'=>'Huawei Browser', 'rekonq'=>'Rekonq', 'kioclient'=>'KIO Client', 'PHX'=>'Phoenix', 'AOL'=>'AOL Exporer', 'AlohaBrowser'=>'Aloha Browser', 'XWEB'=>'XWeb', 'RDDocuments'=>'Readdle Documents', 'BingSapphire'=>'Bing Search App', 'BonEcho'=>'Firefox', 'GranParadiso'=>'FireFox');
        $infos = array_merge($infos, array(
            'browser_title'     => isset($rep[$matches[1]]) ? $rep[$matches[1]] : $matches[1],
            'browser_version'   => isset($matches[2]) ? $matches[2] : '',
        ));
    }

    /* Check no version written browser */
    elseif (preg_match('/(HeadlessChrome|Advanced Browser|Magic Browser|Pinterest|Sunrise|Keepsafe Browser|LBBROWSER|WebexTeams|wpif)/Ui', $ua, $matches)) {
        $rep = array('LBBROWSER'=>'Liebao', 'WebexTeams'=>'Webex Teams');
        $infos = array_merge($infos, array(
            'browser_title'     => isset($rep[$matches[1]])?$rep[$matches[1]]:$matches[1],
        ));
    }

    /* Check some other browsers */
    elseif (preg_match('/(TencentTraveler)/i', $ua, $matches)) {
        $rep = array('TencentTraveler'=>'TT Explorer');
        $infos = array_merge($infos, array(
            'browser_title'     => isset($rep[$matches[1]])?$rep[$matches[1]]:$matches[1],
            'browser_version'   => '',
        ));
    }

    /* Check ecosia browser */
    elseif (preg_match('/\(Ecosia[^@]*@([0-9\.]*)\)/i', $ua, $matches)) {
        $infos = array_merge($infos, array(
            'browser_title'     => 'Ecosia',
            'browser_version'   => isset($matches[1]) ? $matches[1] : '',
        ));
    }

    /* Check gen computer browsers */
    elseif (preg_match('/(Chromium|Netscape|Minefield|Fennec|Firefox|Galeon|iCab|Konqueror|IEMobile|Shiira|NetNewsWire|Symphony|Lynx|Crazy Browser|UCBrowser|GoBrowser|Cobra|ELinks|HotJava|Tor Browser|Dillo|FortiMail|VSE|MultiZilla|UCWEB|Otter|Netbox|AvantGo|NetFront|WhatsApp|Dalvik|Snapchat|ManicTime|Flock|SkypeUrlPreview Preview)(?:\/| )?([a-z0-9\._]*)/i', $ua, $matches)) {
        $rep = array('VSE'=>'VSE Link Tester', 'UCWEB' => 'UC Browser', 'NetFront' => 'Access NetFront', 'SkypeUrlPreview Preview' => 'Skype URL Preview');
        $infos = array_merge(array(
            'platform_type'     => 'computer',
            'browser_title'     => isset($rep[$matches[1]]) ? $rep[$matches[1]] : $matches[1],
            'browser_version'   => str_replace('_', '.', $matches[2]),
        ), $infos);
    }

    /* Check chrome browser - separated from gen case of alternative browser using Chrome user agent */
    elseif (preg_match('/(Chrome)(?:\/| )?([a-z0-9\._]*)/', $ua, $matches)) {
        $infos = array_merge(array(
            'platform_type'     => 'computer',
            'browser_title'     => 'Chrome',
            'browser_version'   => str_replace('_', '.', $matches[2]),
            ), $infos
        );

        /* Amigo's Mail.ru web browser based on chrome (and using chrome version) */
        if (preg_match('/(MRCHROME)/', $ua, $matches))
            $infos['browser_title'] = 'Amigo';
    }

    /* Check gen mobile browsers */
    elseif (preg_match('/(Dorado WAP-Browser|FeedlyApp)(?:\/| )?([a-z0-9\._]*)/', $ua, $matches)) {
        $rep = array('Dorado WAP-Browser'=>'Dorado');
        $infos = array_merge(array(
            'platform_type'     => 'handheld',
            'platform_format'   => 'handheld',
            'browser_title'     => isset($rep[$matches[1]]) ? $rep[$matches[1]] : $matches[1],
            'browser_version'   => str_replace('_', '.', $matches[2]),
            ), $infos
        );
    }

    /* Check directly written browsers */
    elseif (preg_match('/^(PrivacyBrowser)\/([a-z0-9\._]*)$/i', $ua, $matches)) {
        $rep = array();
        $infos = array_merge(array(
            'browser_title'     => isset($rep[$matches[1]]) ? $rep[$matches[1]] : $matches[1],
            'browser_version'   => str_replace('_', '.', $matches[2]),
            ), $infos
        );
    }


    /* Check OmniWeb, before safari */
    elseif (preg_match('/OmniWeb\/v?([^ \.]+)/', $ua, $matches)) {
        $versions = array('622' => '5.9', '626' => '5.11', '630' => '6');
        $infos = array_merge(array(
            'browser_title'     => 'OmniWeb',
            'browser_version'   => array_key_exists($matches[1], $versions) ? $versions[$matches[1]] : $matches[1],
            ), $infos
        );
    }

    /* Check gen browser : ?? */
    elseif (preg_match('/Version\/([a-z0-9\._]*)( Mobile[\/0-9A-Z]*)? Safari\//', $ua, $matches)) {
        if (!isset($infos['system_title']) or $infos['system_title'] == 'Android')
            $infos = array_merge(array(
                'browser_title'     => 'Chrome'.(isset($matches[2]) ? ' Mobile' : ''),
                'browser_version'   => str_replace('_', '.', $matches[1])
                ), $infos
            );
        else
            $infos = array_merge(array(
                'browser_title'     => 'Safari'.(isset($matches[2]) ? ' Mobile' : ''),
                'browser_version'   => str_replace('_', '.', $matches[1])
                ), $infos
            );
    }

    /* Check no version detection browser */
    elseif (preg_match('/(Opera Mobi)/', $ua, $matches))
        $infos = array_merge(array(
            'browser_title'     => $matches[1],
            'browser_version'   => '',
            ), $infos
        );

    /* Check macOS software over CFNetwork */
    elseif (preg_match('/^([A-Za-z0-9\.%\']*)\/[0-9\.]* CFNetwork\//', $ua, $matches)) {
        $app = array('MobileSafari' => 'Safari Mobile');
        $infos = array_merge(array(
            'browser_title'     => isset($app[$matches[1]]) ? $app[$matches[1]] : urldecode($matches[1]),
            'browser_version'   => '',
            ), $infos
        );
    }

    /* Check undeterminated safari version */
    elseif (preg_match('/(Safari|AppleWebKit)/', $ua, $matches)) {
        if (isset($infos['system_title']) and $infos['system_title'] == 'Android') {
            /* Check for version */
            if (preg_match('/Version\/([0-9\.]*)/', $ua, $matches2))
                $version = $matches2[1];
            else
                $version = '';

            $infos = array_merge(array(
                'browser_title'     => 'Android browser',
                'browser_version'   => $version,
                ), $infos
            );
      } else {
            $infos = array_merge(array(
                'browser_title'     => 'Safari compatible',
                'browser_version'   => '',
                ), $infos
            );
        }
    }

    /* Check specific software */
    elseif (preg_match('#^(^(?!Mozilla)[^/]*)/([0-9\.]*)#i', $ua, $matches)) {
        $app = array();
        $infos = array_merge(array(
            'browser_title'     => isset($app[$matches[1]]) ? $app[$matches[1]] : urldecode($matches[1]),
            'browser_version'   => $matches[2],
            ), $infos
        );
    }

    /* Check msie */
    elseif (preg_match('/MSIE ?([a-z0-9\._]*)(|;|\,|\-|\))/', $ua, $matches))
        $infos = array_merge(array(
            'browser_title'     => 'Internet Explorer',
            'browser_version'   => str_replace('_','.',$matches[1]),
            ), $infos
        );

    /* Check IE Mobile detection */
    elseif (preg_match('/(IEMobile)(?:\ )([a-z0-9\._]*)/', $ua, $matches))
        $infos = array_merge(array(
            'browser_title'     => 'Internet Explorer Mobile',
            'browser_version'   => str_replace('_','.',$matches[2]),
            ), $infos
        );

    /* Check trident compatible browsers */
    elseif (preg_match('/(Trident\/)([a-z0-9\._]*)/', $ua, $matches))
        if (preg_match('/rv:11.0/', $ua, $m))
            $infos = array_merge(array(
                'browser_title'     => 'Internet Explorer',
                'browser_version'   => '11.0',
                ), $infos
            );
        else
            $infos = array_merge(array(
                'browser_title'     => 'Trident compatible',
                'browser_version'   => str_replace('_','.',$matches[2]),
                ), $infos
            );

    /* Check America Online Browser */
    elseif (preg_match('/America Online Browser ([a-z0-9\._]*);/', $ua, $matches))
        $infos = array_merge(array(
            'browser_title'     => 'America Online Browser',
            'browser_version'   => str_replace('_','.',$matches[1]),
            ), $infos
        );

    /* Check Windows Media Player */
    elseif (preg_match('/^Windows-Media-Player\/([^\(]*)|Media Player Classic/', $ua, $matches))
        $infos = array_merge(array(
            'browser_title'     => 'Microsoft Windows Media Player',
            'browser_version'   => isset($matches[1])?$matches[1]:'',
            'system_title'      => 'Windows',
            ), $infos
        );


    /* Check FrontPage */
    elseif (preg_match('/^MSFrontPage\/([^\(]*)/', $ua, $matches))
        $infos = array_merge(array(
            'browser_title'     => 'Microsoft FrontPage',
            'browser_version'   => $matches[1],
            'system_title'      => 'Windows',
            ), $infos
        );

    /* Check MoonBrowser */
    elseif (preg_match('/MoonBrowser \(([^\(]*)\)/', $ua, $matches))
        $infos = array_merge(array(
            'browser_title'     => 'Moon Browser',
            'browser_version'   => $matches[1],
            'system_title'      =>'Windows',
            ), $infos
        );

    /* Check Barca */
    elseif (preg_match('/^Barca\/([^\(]*)/', $ua, $matches))
        $infos = array_merge(array(
            'browser_title'     => 'Barca',
            'browser_version'   => $matches[1],
            'system_title'      => 'Windows',
            ), $infos
        );

    /* Check Pizilla */
    elseif (preg_match('/^Pizilla\+\+ ver ([0-9\. ]*)$/', $ua, $matches))
        $infos = array_merge(array(
            'browser_title'     => 'Pizilla',
            'browser_version'   => $matches[1],
            ), $infos
        );

    /* Check Microsoft Data Access Internet Publishing Provider DAV */
    elseif (preg_match('/^Microsoft Data Access Internet Publishing Provider DAV$/', $ua, $matches))
        $infos = array_merge(array($infos,
            'browser_title'     => 'Office (2000 and superior) WebDav client ',
            'system_title'      => 'Windows',
        ));

    /* Check Microsoft Office */
    elseif (preg_match('/^(Microsoft Office [A-Za-z0-9\ ]*)$/', $ua, $matches))
        $infos = array_merge(array($infos,
            'browser_title'     => $matches[1],
        ));
    elseif (preg_match('/ms-office/', $ua, $matches))
        $infos = array_merge(array($infos,
            'browser_title'     => 'Microsoft Office',
        ));
    /* Check HotJava */
    elseif (preg_match('/^HotJava\/([^\(]*)/', $ua, $matches))
        $infos = array_merge(array(
            'browser_title'     => 'HotJava',
            'browser_version'   => $matches[1],
            ), $infos
        );

    /* Check MAUI WAP Browser  */
    elseif (preg_match('/^MAUI(|_)WAP(|_)Browser/', $ua, $matches))
        $infos = array_merge(array(
            'browser_title'     => 'MAUI WAP Browser',
            'browser_version'   => '',
            'platform_type'     => 'handheld',
            'platform_format'   => 'handheld',
            ), $infos
        );

    /* Check other browsers with specific details */
    elseif (preg_match('/(AWeb|Blazer|DreamPassport|Liberate DTV|SlimBrowser|Zoo Tycoon 2 Client)(?:\/| )?([a-z0-9\._]*)?/i', $ua, $matches)) {
        $rep = array(
            'AWeb'                  => array('system_title'=>'Amiga'),
            'Blazer'                => array('platform_company'=>'Palm', 'system_title'=>'Palm OS', 'platform_type' => 'handheld', 'platform_format' => 'handheld',),
            'Liberate DTV'          => array('platform_company'=>'LiberateHealth',),
            'DreamPassport'         => array('platform_company'=>'Sega', 'platform_model'=>'DreamCast', 'system_title'=>'DreamCast OS'),
            'SlimBrowser'           => array('system_title' =>'Windows'),
            'Zoo Tycoon 2 Client'   => array('system_title' =>'Windows', 'browser_title'=>'Microsoft Zoo Tycoon 2 Client'),
        );
        $infos = array_merge($infos, array(
            'browser_title'     => isset($matches[1])?$matches[1]:'',
            'browser_version'   => isset($matches[2])?$matches[2]:'',
            ), (isset($rep[$matches[1]])?$rep[$matches[1]]:array())
        );
    }

    /* Check Gecko compatible */
    elseif (preg_match('/Gecko/', $ua, $matches))
        $infos = array_merge(array(
            'browser_title'     => 'Firefox compatible',
            ), $infos
        );

    /* Check old netscape */
    elseif (preg_match('/^Mozilla\/([1-4].[0-9]{1,2})/', $ua, $matches))
        $infos = array_merge(array(
            'browser_title'     => 'Netscape',
            'browser_version'   => $matches[1],
            ), $infos
        );

    /* set type to "visitor" if not defined and browser title set */
    if (!isset($infos['platform_type']) and isset($infos['browser_title']) and $infos['browser_title'] != '')
        $infos['platform_type'] = 'computer';
    if ((!isset($infos['platform_format']) or $infos['platform_format'] == '') and isset($infos['browser_title']) and $infos['browser_title'] != '')
        $infos['platform_format'] = 'desktop';

    if (!isset($infos['platform_format']) or $infos['platform_format'] == '' or $infos['platform_format'] == 'desktop') {
        /* Check for generic mobile */
        if (preg_match('/mobile/i', $ua, $matches))
            $infos = array_merge($infos,array(
            'platform_format'   => 'handheld',
        ));

        /* Check for generic tablet */
        elseif (preg_match('/tablet/i', $ua, $matches))
            $infos = array_merge($infos, array(
                'platform_format'   => 'tablet',
            ));
    }

    /* Check browser and version */
    if (isset($infos['browser_title']) and _client_checkSoftwareIsFake('browser', $infos, $ua))
        return $fake;

    /* return result */
    return _client_mergeUAInfos($req, $infos);
}

/**
 * Return bot index if bot.
 * @param string $ua
 * @return mixed
 */
function client_checkIsBot($ua=false) {
    /* get user agent */
    if (!$ua) $ua = client_getUserAgent();

    /* Load known bots */
    $knownBots = client_getKnownBots();

    /* build detection pattern */
    $detectionPattern = implode('|', array_keys($knownBots));
    $detectionPattern = string_escape4Regex($detectionPattern);
    $detectionPattern = '/'.$detectionPattern.'/i';

    /* start detection and return result */
    if (preg_match($detectionPattern, strtolower($ua), $result))
        return $result[0];
    return false;
}

/**
 * Return bot info if bot.
 * @param string $bot
 * @return mixed
 */
function client_getBotInfos($bot='') {
    $knownBots = client_getKnownBots();

    if (isset($knownBots[$bot]))
        return $knownBots[$bot];
    return array();
}


/**
 * Return bot info from user agent.
 * @param string $ua
 * @return mixed
 */
function client_getBotInfosFromUserAgent($ua='') {
    $knownBots = client_getKnownBots();

    /* build detection pattern */
    $detectionPattern = implode('|', array_keys($knownBots));
    $detectionPattern = string_escape4Regex($detectionPattern);
    $detectionPattern = '/'.$detectionPattern.'/i';

    /* start detection and return result */
    if (preg_match ($detectionPattern, strtolower($ua), $result ) )
        return array_merge(array('bot_name'=>$result[0]), $knownBots[$result[0]]);
    return false;
}

/**
 * Return known bots list.
 * @return array
 */
function client_getKnownBots() {
    include_once('clientBots/Bots.php.inc');
    return _client_getKnownBots();
}

/* Tool methods */

/**
 * return tool index if tool
 * @param string $ua
 * @return mixed
 */
function client_checkIsTool($ua=false) {
    /* get user agent */
    if (!$ua) $ua = client_getUserAgent();

    /* Load known tools */
    $knownTools = client_getKnownTools();

    /* build detection pattern */
    $detectionPattern = implode('|', array_keys($knownTools));
    $detectionPattern = string_escape4Regex($detectionPattern);
    $detectionPattern = '/'.$detectionPattern.'/i';

    /* start detection and return result */
    if (preg_match($detectionPattern, strtolower($ua), $result))
        return $result[0];
    return false;
}

/**
 * Return tool info if tool.
 * @param string $tool
 * @return mixed
 */
function client_getToolInfos($tool='') {
    $knownTools = client_getKnownTools();

    if (isset($knownTools[$tool]))
        return $knownTools[$tool];
    return array();
}

/**
 * Return tool info from user agent.
 * @param string $ua
 * @return mixed
 */
function client_getToolInfosFromUserAgent($ua='') {
    $knownTools = client_getKnownTools();

    /* build detection pattern */
    $detectionPattern = implode('|', array_keys($knownTools));
    $detectionPattern = string_escape4Regex($detectionPattern);
    $detectionPattern = '/'.$detectionPattern.'/i';

    /* start detection and return result */
    if (preg_match($detectionPattern, strtolower($ua), $result))
        return array_merge(array('tool_name'=>$result[0]), $knownTools[$result[0]]);
    return false;
}

/**
 * Return given known tool list.
 * @return array
 */
function client_getKnownTools() {
    include_once('clientBots/Tools.php.inc');
    return _client_getKnownTools();
}

/* Before bots methods */

/**
 * Return bot index if before bot.
 * @param string $ua
 * @return mixed
 */
function client_checkIsBeforeBot($ua=false) {
    /* get user agent */
    if (!$ua) $ua = client_getUserAgent();

    /* Load known bots */
    $knownBots = client_getKnownBeforeBots();

    /* build detection pattern */
    $detectionPattern = implode('|', array_keys($knownBots));
    $detectionPattern = string_escape4Regex($detectionPattern);
    $detectionPattern = '/'.$detectionPattern.'/i';

    /* start detection and return result */
    if (preg_match($detectionPattern, strtolower($ua), $result))
        return $result[0];
    return false;
}

/**
 * Return bot info if bot.
 * @param string $bot
 * @return mixed
 */
function client_getBeforeBotInfos($bot='') {
    $knownBots = client_getKnownBeforeBots();

    if (isset($knownBots[$bot]))
        return $knownBots[$bot];
    return array();
}


/**
 * Return bot info from user agent.
 * @param string $ua
 * @return mixed
 */
function client_getBeforeBotInfosFromUserAgent($ua='') {
    $knownBots = client_getKnownBeforeBots();

    /* build detection pattern */
    $detectionPattern = implode('|', array_keys($knownBots));
    $detectionPattern = string_escape4Regex($detectionPattern);
    $detectionPattern = '/'.$detectionPattern.'/i';

    /* start detection and return result */
    if (preg_match ($detectionPattern, strtolower($ua), $result ) )
        return array_merge(array($knownBots[$result[0]]['platform_type'].'_name'=>$result[0]), $knownBots[$result[0]]);
    return false;
}

/**
 * Return known bots list.
 * @return array
 */
function client_getKnownBeforeBots() {
    include_once('clientBots/Before.php.inc');
    return _client_getKnownBeforeBots();
}

/* Unknown bots methods */

/**
 * Return unknown index if unknown.
 * @param string $ua
 * @return mixed
 */
function client_checkIsUnknown($ua=false) {
    /* get user agent */
    if (!$ua) $ua = client_getUserAgent();

    /* Load known unknowns */
    $knownUnknowns = client_getKnownUnknowns();

    /* build detection pattern */
    $detectionPattern = implode('|', array_keys($knownUnknowns));
    $detectionPattern = string_escape4Regex($detectionPattern);
    $detectionPattern = '/'.$detectionPattern.'/i';

    /* start detection and return result */
    if (preg_match($detectionPattern, strtolower($ua), $result))
        return $result[0];
    return false;
}

/**
 * Return unknown info if unknown.
 * @param string $unknown
 * @return mixed
 */
function client_getUnknownInfos($unknown='') {
    $knownUnknowns = client_getKnownUnknowns();

    if (isset($knownUnknowns[$unknown]))
        return $knownUnknowns[$unknown];
    return array();
}

/**
 * Return unknown info from user agent.
 * @param string $ua
 * @return mixed
 */
function client_getUnknownInfosFromUserAgent($ua='') {
    $knownUnknowns = client_getKnownUnknowns();

    /* build detection pattern */
    $detectionPattern = implode('|', array_keys($knownUnknowns));
    $detectionPattern = string_escape4Regex($detectionPattern);
    $detectionPattern = '/'.$detectionPattern.'/i';

    /* start detection and return result */
    if (preg_match($detectionPattern, strtolower($ua), $result))
        return array_merge(array('bot_name'=>$result[0]), $knownUnknowns[$result[0]]);
    return false;
}

/**
 * Return given known unknown list.
 * @return array
 */
function client_getKnownUnknowns() {
    include_once('clientBots/Unknowns.php.inc');
    return _client_getKnownUnknowns();
}

/*** generic tool/boot specific methods */

/**
 * Return generic index if generic.
 * @param string $ua
 * @return mixed
 */
function client_checkIsGeneric($ua=false) {
    /* get user agent */
    if (!$ua) $ua = client_getUserAgent();

    /* Load known generic agents */
    client_loadKnownGenerics();
    $knownGenerics = _client_getKnownGenericsL1();

    /* build detection pattern */
    $detectionPattern = implode('|', array_keys($knownGenerics));
    $detectionPattern = string_escape4Regex($detectionPattern);
    $detectionPattern = '/'.$detectionPattern.'/i';

    /* start detection and return result */
    if (preg_match ($detectionPattern, strtolower($ua), $result))
        return $result[0];

    /* Load known generic agents */
    $knownGenerics = _client_getKnownGenericsL2();

    /* build detection pattern */
    $detectionPattern = implode('|', array_keys($knownGenerics));
    $detectionPattern = string_escape4Regex($detectionPattern);
    $detectionPattern = '/'.$detectionPattern.'/i';

    /* start detection and return result */
    if (preg_match ($detectionPattern, strtolower($ua), $result))
        return $result[0];
    return false;
}

/**
 * Return generic info if generic.
 * @param string $generic
 * @return mixed
 */
function client_getGenericInfos($generic='') {
    /* Load known generic agents */
    client_loadKnownGenerics();
    $knownGenerics = _client_getKnownGenericsL1();

    if (isset($knownGenerics[$generic]))
        return $knownGenerics[$generic];

    /* Load known generic agents */
    $knownGenerics = _client_getKnownGenericsL2();

    if (isset($knownGenerics[$generic]))
        return $knownGenerics[$generic];
    return array();
}

/**
 * Return bot info from user agent.
 * @param string $ua
 * @return mixed
 */
function client_getGenericInfosFromUserAgent($ua='') {

    /* remove hotjava from java results */
    if (preg_match('/^HotJava/i', $ua))
        return false;

    /* Load known generic agents */
    client_loadKnownGenerics();
    $knownGenerics = _client_getKnownGenericsL1();

    /* build detection pattern */
    $detectionPattern = implode('|', array_keys($knownGenerics));
    $detectionPattern = string_escape4Regex($detectionPattern);
    $detectionPattern = '/'.$detectionPattern.'/i';

    /* start detection and return result */
    if (preg_match ($detectionPattern, strtolower($ua), $result)) {
        $bot = $knownGenerics[$result[0]];
        return array_merge(array($bot['platform_type'].'_name'=>$result[0]), $bot);
    }

    /* Load known generic agents */
    $knownGenerics = _client_getKnownGenericsL2();

    /* build detection pattern */
    $detectionPattern = implode('|', array_keys($knownGenerics));
    $detectionPattern = string_escape4Regex($detectionPattern);
    $detectionPattern = '/'.$detectionPattern.'/i';

    /* start detection and return result */
    if (preg_match ($detectionPattern, strtolower($ua), $result)){
        $bot = $knownGenerics[$result[0]];
        return array_merge(array($bot['platform_type'].'_name'=>$result[0]), $bot);
    }
    return false;
}

/**
 * Load known web generic library lists.
 * @return void
 */
function client_loadKnownGenerics() {
    include_once('clientBots/Generics.php.inc');
}

/*** hack type specific methods ***/

/**
 * Return hack index if hack.
 * @param string $ua
 * @return mixed
 */
function client_checkIsHack($ua=false) {
    /* get user agent */
    if (!$ua) $ua = client_getUserAgent();

    /* Load known bots */
    $knownHacks = client_getKnownHacks();

    /* build detection pattern */
    $detectionPattern = implode('|', array_keys($knownHacks));
    $detectionPattern = string_escape4Regex($detectionPattern);
    $detectionPattern = '/'.$detectionPattern.'/i';

    /* start detection and return result */
    if (preg_match($detectionPattern, strtolower($ua), $result))
        return $result[0];
    return false;
}

/**
 * Return bot info if hack.
 * @param string $hack
 * @return mixed
 */
function client_getHackInfos($hack='') {
    $knownHacks = client_getKnownHacks();

    if (isset($knownHacks[$hack]))
        return $knownHacks[$hack];
    return array();
}

/**
 * Return hack info from user agent.
 * @param string $ua
 * @return mixed
 */
function client_getHackInfosFromUserAgent($ua='') {
    $knownHacks = client_getKnownHacks();

    /* build detection pattern */
    $detectionPattern = implode('|', array_keys($knownHacks));
    $detectionPattern = string_escape4Regex($detectionPattern);
    $detectionPattern = '/'.$detectionPattern.'/i';

    /* start detection and return result */
    if (preg_match($detectionPattern, strtolower($ua), $result))
        return $knownHacks[$result[0]];
    return false;
}

/**
 * Return given known web hack or hack list.
 * @return array
 */
function client_getKnownHacks() {
    include_once('clientBots/Hacks.php.inc');
    return _client_getKnownHacks();
}

/**
 * Return fake visitor info from user agent.
 * @param string $ua
 * @return mixed
 */
function client_getFakeInfosFromUserAgent($ua='') {
    $fake = array('platform_type'     => 'fake');

    $ua = trim($ua);

    if ($ua == '')
        return $fake;

    if (preg_match('#^Mozilla$#i', $ua))
        return $fake;

    if (preg_match('#^Mozilla-1.1$#i', $ua))
        return $fake;

    if (preg_match('#^Mozilla[/| ][0-9][.|,][0-9]$#i', $ua))
        return $fake;

    if (preg_match('#^Mozilla/%2F[0-9][.|,]0$#i', $ua))
        return $fake;

    if (preg_match('#^Mozilla/[0-9][.|,]0 \(compatible;?\)?$#i', $ua))
        return $fake;

    if (preg_match('#^Mozilla/[0-9].0 \(compatible; MSIE[0-9].00; Windows 200[1-9]\)#i', $ua))
        return $fake;

    if (preg_match('#^Mozilla/4.0 \(compatible; MSIE 5.0; Windows 98\)#i', $ua))
        return $fake;

    if (preg_match('#^Mozilla/4.0 \(compatible; MSIE 999.1; Unknown\)#i', $ua))
        return $fake;

    if (preg_match('#Mozilla/4.0 \(compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.0\) Opera 7.02 Bork-edition \[en\]#i', $ua))
        return $fake;

    if (preg_match('#Mozilla/5.0 \(compatible; MSIE 10.0; Macintosh; Intel Mac OS X 10_7_3; Trident/6.0\)#i', $ua))
        return $fake;

    /* fake russian spammer */
    if (preg_match('#Mozilla/4.0 \(compatible; MSIE 7.0b; Windows NT 6.0 ; .NET CLR 2.0.50215; SL Commerce Client v1.0; Tablet PC 2.0#i', $ua))
        return $fake;

    /* fake russian spammer */
    if (preg_match('#Mozilla/4.0 \(compatible; MSIE 5.0; Windows 3.1\)#i', $ua))
        return $fake;

    /* fake russian spammer */
    if (preg_match('#Mozilla/4.0 \(compatible; MSIE 5.0; Windows NT; DigExt\)#i', $ua))
        return $fake;

    /* fake russian spammer */
    if (preg_match('#Mozilla/4.0 \(compatible; MSIE 6.0; AOL 9.0; Windows NT 5.1\)#i', $ua))
        return $fake;

    /* fake requests from Amazon AWS services */
    if (preg_match('#^Mozilla/5.0 \(Windows NT 6.1; rv:15.0\) Gecko/20120716 Firefox/15.0a2$#i', $ua))
        return $fake;
    if (preg_match('#^"Mozilla/5.0([^"]*)"$#i', $ua))
        return $fake;
    if (preg_match('#^"Mozilla/5.0$#i', $ua))
        return $fake;
     if (preg_match('#"$#i', $ua))
        return $fake;

    /* fake requests from Lituania */
    if (preg_match('#^Dalvik/2.1.0 \(Linux; U; Android 5.1.1; D5503 Build/14.6.A.1.236\)$#i', $ua))
        return $fake;

    /* fake requests with double user agent combined */
    if (preg_match_all('#Mozilla/[0-9].0#i', $ua, $matches) and count($matches[0]) > 1)
        return $fake;

    /* fake requests from China */
    if (preg_match('#^MSDN SurfBear$#i', $ua))
        return $fake;

    /* fake request with ' around */
    if (preg_match('#^\'(.*)\'$#', $ua))
        return $fake;

    /* fake requests from 80.169.91.26 */
    if (preg_match('#-[0-9]{7}$#', $ua))
        return $fake;

    /* badly set user agent */
    if (preg_match('#^=#', $ua))
        return $fake;

    /* fake user agent */
    if (preg_match('#Firefox/2.0.0.[0-9]+ Navigator/9.0#', $ua))
        return $fake;

    /* badly set user agent */
    if (preg_match('#^User-Agent:#i', $ua))
        return $fake;

    /* php default useragent passed as string intead of CONST */
    if (preg_match('#^_USERAGENT_$#i', $ua))
        return $fake;

    /* bad msie/msid request */
    if (preg_match('#^Mozilla/4.0 \(compatible; MSID [0-9\.]+; Windows NT [0-9\.]+\)$#i', $ua))
        return $fake;

    /* bad msie/msid request */
    if (preg_match('#^Mozilla/5.[0-9](...)?$#i', $ua))
        return $fake;

    /* 1 or 2 letters UA like "Z" or "lp" */
    if (preg_match('#^[a-zA-Z0-9]{1,2}$#i', $ua))
        return $fake;

    /* Bad chrome version */
    if (preg_match('#Chrome/0.A.B.C#i', $ua))
        return $fake;

    /* Bad word version */
    if (preg_match('#Word/0.0.0$#i', $ua))
        return $fake;

    /* Fake chrome ua */
    if (preg_match('#^Chrome/71.0.3578.98$#i', $ua))
        return $fake;

    /* Check for user agent where spaces are replaced by plus signs */
    if (($chars = count_chars($ua, 1)) and isset($chars[ord('+')]) and $chars[ord('+')] >= 10 and (!isset($chars[ord(' ')])))
        return $fake;

    /* badly set user agent */
    if (preg_match('#Connection: closern$#i', $ua))
        return $fake;

    /* Fake visits from AWS */
    if (preg_match('#^Mozilla/5.0 Firefox/33.0$#i', $ua))
        return $fake;
    if (preg_match('#^Mozilla/5.0 \(Macintosh; Intel Mac OS X x.y; rv:42.0\) #i', $ua))
        return $fake;
    if (preg_match('#^Mozilla/5.0 \(Windows NT 6.1; Win64; x64; rv:47.0\) #i', $ua))
        return $fake;
    if (preg_match('#^Mozilla/5.0 \(X11; Linux x86_64\) (.*)(Trident/5.0|Firefox/42.0|OPR/38.0.2220.41|Gecko/20100101 Safari/602.1|AppleWebKit/537.36 \(KHTML, like Gecko\) Safari/602.1)#i', $ua))
        return $fake;

    /* Mispelled Firefox user agent, exepting BrandVerity which is using a badly set user agent *
    if (preg_match('#^Mozilla/5.0 \(([^\)]*) rv:([0-9a-zA-Z\.]*)\) ([^\)]*) Firefox/([0-9a-zA-Z\.]*)((?!BrandVerity).)*$#i', $ua, $matches) and ($matches[2] != $matches[4]) and (version_compare($matches[4], 5, '>=')))
        return $fake;
    */
    /* Fake Firefox user agent, with strange version identifier */
    if (preg_match('#^Mozilla/5.0 (.*) (Firefox|Safari)/([0-9A-Z]{4,6})$#i', $ua))
        return $fake;
    if (preg_match('#^Mozilla/5.0 (.*) Firefox/([0-9\.]*)/([^ ]*)$#i', $ua))
        return $fake;
    if (preg_match('#^Mozilla/5.0 (.*) fer6v8QG-4 Firefox/([0-9\.]*)$#i', $ua))
        return $fake;

    /* Real Firefox user agent used for fake requests */
    if (preg_match('#^Mozilla/5.0 \(Windows NT 6.1; WOW64; rv:40.0\) Gecko/20100101 Firefox/40.1$#i', $ua))
        return $fake;

    /* Fake Firefox (Chrome) user agent */
    if (preg_match('#^Mozilla/5.0 \(X11; Linux x86_64; rv:10.0\) Gecko/20150101 Firefox/[0-9]{2}.0 \(Chrome\)$#i', $ua))
        return $fake;

    /* Spams */
    if (preg_match('#^Mozilla/5.0 \(X11; Linux x86_64; rv:109\)$#i', $ua))
        return $fake;

    /* fake visits *
    if (preg_match('#UserAgent-Cube$#i', $ua))
        return $fake;
    */

    /* Fake visits with text before "Mozilla 5.0" */
    if (preg_match('#^(User-Agent|:|Mac|")(.*)Mozilla/5.0#Ui', $ua))
        return $fake;

    /* Fake visits, sample : Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko RqIwg8izYw6xskukQQDh */
    if (preg_match('#(.{40,})([0-9a-zA-Z]{16,24})$#', $ua))
        return $fake;

    /* Fake visits, with strange Edge version */
    if (preg_match('#^Mozilla/5.0 \(Windows NT 10.0; Win64; x64\) AppleWebKit/537.36 \(KHTML, like Gecko\) Chrome/[0-9\.]* Safari/537.36 Edge/[A-Z0-9]{8}$#', $ua))
        return $fake;

    /* Fake visits, with dumb/single word ua */
    if (preg_match('#^(Foo|default|Google|googlebot|AGENT|null|url)$#i', $ua))
        return $fake;

    /* Fake visit, with dumb ua */
    if (preg_match('#^Mozilla/5.0 \(admin\)$#', $ua))
        return $fake;

    /* Fake visit, without any space at the beginning of Android5.1.1/Oppo A33 device | Opera/12 */
    if (preg_match('#^(Mozilla|Opera)/([0-9\.]*)\(#', $ua))
        return $fake;

    /* Fake visit, from switzerland */
    if (preg_match('#like Gecko 4tds562$#', $ua))
        return $fake;

    /* Fake visit, with firefox addon */
    if (preg_match('#Firefox/[0-9\.]+ tkx1yjj66$#', $ua))
        return $fake;

    /* Fake visit, from finland */
    if (preg_match('#\- 111512\-1801120036\-1\.44$#', $ua))
        return $fake;

    /* Fake facebook external hit, from germany */
    if (preg_match('#^acebookexternalhit#', $ua))
        return $fake;

    /* Fake firefox/opera version */
    if (preg_match('#Firefox/4.0 Opera 12.14$#', $ua))
        return $fake;

    /* Fake IE with additional data */
    if (preg_match('#APCPMS=#', $ua))
        return $fake;

    /* Hack ua using HTML script tag */
    if (preg_match('#text/javascript#', $ua))
        return $fake;

    /* Fake UA with strange caracters in front of mozilla */
    if (preg_match('#^b\'Mozilla#', $ua))
        return $fake;

    /* Fake Mozilla version */
    if (preg_match('#^Mozilla/(?![4|5]\.0)#', $ua))
        return $fake;

    /* Fake Mediapartners-Google */
    if (preg_match('#Mediapartners\-Google\) Chrome/\[CHROME_VERSION\] Safari/\[WEBKIT_VERSION\]$#', $ua))
        return $fake;

    /* Fake IE11 UA : Mozilla/5.0 (csriveraine; Windows NT 10.0; WOW64; Trident/7.0; rv:11.0) like Gecko */
    if (preg_match('#^Mozilla/5.0 \(csriveraine;#', $ua))
        return $fake;

    /* Fake IE11 UA : Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko,411129,&quot;3.674257&quot; */
    if (preg_match('#,411129,&quot;3.674257&quot;$#', $ua))
        return $fake;

    /* Fake UA      : Mozilla/5.0 (Windows NT 10.0; WOW64; Trident/7.0; rv:11.0) like Gecko/q9LVT10o-7 */
    /* Fake UA      : Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.61 Safari/537.36/iYA9HebN-7 */
    /* Fake UA      : Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/85.0.4183.83 Safari/537.36/9eeyPJns-18 */
    /* Fake UA      : Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/85.0.4183.83 Safari/537.36/GXSim7tn-45 */
    /* Fake UA      : Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/85.0.4183.83 Safari/537.36/6zL1MWua-33 */
    /* Fake UA      : Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:80.0) Gecko/20100101 Firefox/80.0/3ksI6CtQ-58 */
    /* Fake UA      : Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36 Edg/81.0.416.88/8mqNiKuL-99 */
    /* Not Fake UA  : Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36/9nlzYpuG-21 */
    /* Not Fake UA  : Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.77 Safari/537.36/4MEeS4T2-48 */
    if (preg_match('#(Gecko|Firefox/[0-9\.]*|Safari/537.36)/(?=.*[0-9])(?=.*[A-Z])(?=.*[a-z])([a-zA-Z0-9_]+)\-[0-9]$#', $ua))
        return $fake;

    /* Fake UA      : Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:80.0) Gecko/20100101 Firefox/80.0 ukymizvpxq6 */
    /* Fake UA      : Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:80.0) Gecko/20100101 Firefox/80.0 r0qkezrsr0 */
    if (preg_match('#Firefox/80.0 [a-z0-9]{10,11}$#', $ua))
        return $fake;

    /* Fake UA      : Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:80.0) Gecko/20100101 lQHGDnYu-22 Firefox/80.0 */
    if (preg_match('#Gecko/20100101 ([a-zA-Z]{8}\-[0-9]{2}) Firefox/([0-9\.]{4,})$#', $ua))
        return $fake;

    /* Fake UA - Advertising through UA */
    if (preg_match('#scaninfo@expanseinc.com$#', $ua))
        return $fake;

    /* Fake visit with uncomplete safari version */
    if (preg_match('#Safari/537\.3$#', $ua))
        return $fake;

    /* Mispelled UA, followed by numerous requests */
    if (preg_match('#KHTML, live Gecko#', $ua))
        return $fake;

    /* Mispelled UA, missing Gecko */
    if (preg_match('#KHTML, like\)#', $ua))
        return $fake;

    /* Fake visit with UA description included */
    if (preg_match('#^Linux / Firefox [0-9]{2,}: Mozilla#', $ua))
        return $fake;

    /* Fake googlebot-image */
    if (preg_match('#^Googlebot-Image$#', $ua))
        return $fake;

    /* Fake googlebot */
    if (preg_match('#Chrome/W\.X\.Y\.Z! Mobile Safari/537.36 \(compatible; Googlebot/#', $ua))
        return $fake;

    /* Undefined client */
    if (preg_match('# NCLIENT[0-9]{2}_[A-Z]*#', $ua))
        return $fake;

    /* Fake security bot */
    if (preg_match('#^Mozilla/5.0 \(Windows NT 6.1; WOW64; Host: #', $ua))
        return $fake;

    /* Badly written ua */
    if (preg_match('#^like Gecko\) #', $ua))
        return $fake;

    /* ALittle Client */
    if (preg_match('#^ALittle Client$#', $ua))
        return $fake;

    /* Badly written ua */
    if (preg_match('#^MMozilla/5\.0 #', $ua))
        return $fake;

    /* default result */
    return false;
}

/*** private methods ***/

/**
 * Return merged informations.
 * @param array $req
 * @param array $infos
 * @return array
 * @private
 */
function _client_mergeUAInfos($req, $infos) {
    if ($req === true)
        $req = array(
            'platform_company', 'platform_country', 'platform_model', 'platform_type', 'platform_format', 'platform_year',
            'system_title', 'system_version',
            'browser_title', 'browser_version'
        );
    $i = array();
    foreach($req as $r)
        if (isset($infos[$r]))
            $i[$r] = $infos[$r];
        else
            $i[$r] = '';
    return $i;
}

/**
 * Load some defined properties for softwares.
 * @return void
 * @private
 */
function _client_loadSoftwareProperties() {
    global $softwareProperties;

    if (isset($softwareProperties))
        return;

    /* Build main list */
    $softwareProperties = array(
        'system' => array(
            'Mac OS X'          => array(
                'MaxVersion'    => '10.12.6',               // final is 10.12.6
                'FakeVersions'  => array(),
                'HasNoVersion'  => true,
                'CheckMethod'   => '',
            ),
            'macOS'          => array(
                'MaxVersion'    => '17.0.0',                // current is 14
                'FakeVersions'  => array(),
                'HasNoVersion'  => false,
                'CheckMethod'   => '',
            ),
            'Windows'           => array(
                'MaxVersion'    => '',
                'FakeVersions'  => array('NT 7.0', 'NT 7.1', 'NT 8.0', 'NT 8.1', 'NT 9.0', 'NT 9.1', 'NT 10.1', 'NT 15.0', 'NT 10.0N', 'NT 6.2.9200.0', 'NT 17.2'),
                'HasNoVersion'  => false,
                'CheckMethod'   => '',
            ),
            'iOS'               => array(
                'MaxVersion'    => '20',                    // current is 17.x
                'FakeVersions'  => array(),
                'HasNoVersion'  => true,
                'CheckMethod'   => '',
            ),
            'Android'           => array(
                'MaxVersion'    => '17',                    // current is 14
                'FakeVersions'  => array('7.8.9'),
                'HasNoVersion'  => true,
                'CheckMethod'   => '',
            ),
        ),
        'browser' => array(
            'Firefox'           => array(
                'MaxVersion'    => '128',                    // current is 120 , nightly is +6
                'FakeVersions'  => array('27.3', '45.62.15', '52.53.90'),
                'HasNoVersion'  => false,
                'CheckMethod'   => '_client_checkFirefoxIsFake',
            ),
            'Chrome'            => array(
                'MaxVersion'    => '125',                    // current is 119
                'FakeVersions'  => array(),
                'HasNoVersion'  => true,
                'CheckMethod'   => '',
            ),
            'Opera'             => array(
                'MaxVersion'    => '111',                    // current is 105
                'FakeVersions'  => array(),
                'HasNoVersion'  => false,
                'CheckMethod'   => '',
            ),
            'Safari'            => array(
                'MaxVersion'    => '22',                    // current is 17.1
                'FakeVersions'  => array(),
                'HasNoVersion'  => true,
                'CheckMethod'   => '',
            ),
            'Internet Explorer' => array(
                'MaxVersion'    => '11.0.50',               // latest
                'FakeVersions'  => array('9.11'),
                'HasNoVersion'  => false,
                'CheckMethod'   => '',
            ),
        ),
    );

    /* Add duplicate */
    $softwareProperties['browser']['Chromium'] = $softwareProperties['browser']['Chrome'];  // Chromium matches Chrome last and fake versions
    $softwareProperties['browser']['Edge'] = $softwareProperties['browser']['Chrome'];      // Edge now matches Chrome last and fake versions
}

/**
 * Check if sofware is fake.
 * @param string $type
 * @param array $infos
 * @param string $ua
 * @return boolean
 */
function _client_checkSoftwareIsFake($type, $infos, $ua) {
    global $softwareProperties;

    /* Load properties */
    _client_loadSoftwareProperties();

    /* Load vars */
    $software = $infos[$type.'_title'];
    $version = isset($infos[$type.'_version']) ? $infos[$type.'_version'] : '';

    /* Check type */
    if (!isset($softwareProperties[$type]))
        return false;

    /* Check software */
    if (!isset($softwareProperties[$type][$software]))
        return false;

    /* Check version */
    if ($softwareProperties[$type][$software]['HasNoVersion'] === false and $version === '')
        return true;

    /* Check max version */
    if ($softwareProperties[$type][$software]['MaxVersion'] != '' and version_compare($version, $softwareProperties[$type][$software]['MaxVersion'], '>=') > 0)
        return true;

    /* Check fake versions */
    if (in_array($version, $softwareProperties[$type][$software]['FakeVersions']))
        return true;

    /* Execute check method */
    if ($softwareProperties[$type][$software]['CheckMethod'] != '' and function_exists($softwareProperties[$type][$software]['CheckMethod']) and call_user_func($softwareProperties[$type][$software]['CheckMethod'], $infos, $ua)) {
        return true;
    }

    /* Return default result */
    return false;
}

/**
 * Check if given UA is a OnePlus device
 * @param string $ua
 * @return boolean result
 * @private
 */
function _client_isOnePlusDevice($ua='') {
    if (preg_match('/ ONE(PLUS)? ?([0-9A-Za-z ]+)( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/OnePlus.php.inc');
        return trim($matches[2]);
    } elseif (preg_match('/ (A|E)([0-9]{4})( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/OnePlus.php.inc');
        if (is_OnePlusKnownDevice($matches[1].$matches[2]))
          return $matches[1].$matches[2];
    } elseif (preg_match('/; (GM|HD|IN|AC|BE|KB|LE)([0-9]{4})( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/OnePlus.php.inc');
        if (is_OnePlusKnownDevice($matches[1].$matches[2]))
          return $matches[1].$matches[2];
    }
    return false;
}

/**
 * Check if given UA is a Huawei device
 * @param string $ua
 * @return boolean result
 * @private
 */
function _client_isHuaweiDevice($ua='') {
    if (preg_match('/ SPH\-[A-Z0-9]{3,4}( Build|\)|;)/', $ua, $matches))
        return false;
    if (preg_match('/ ([A-Z][A-Z0-9]{2,3}\-[A-Z][a-zA-Z0-9]{2,5})( Build|\)|;)/', $ua, $matches)) {
        include_once('clientDevices/Huawei.php.inc');
        return trim($matches[1]);
    } elseif (preg_match('/ (MediaPad [^\)]*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Huawei.php.inc');
        return trim($matches[1]);
    } elseif (preg_match('/Build\/(HONOR|HUAWEI|ORINOQUIA)([A-Z0-9\-]{3,})/i', $ua, $matches)) {
        include_once('clientDevices/Huawei.php.inc');
        return trim($matches[2]);
    } elseif (preg_match('/; HUAWEI (.*)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Huawei.php.inc');
        return trim($matches[1]);
    } elseif (preg_match('/ (T1-A21L|hi6210sft|Che2-L11)/i', $ua, $matches)) {
        include_once('clientDevices/Huawei.php.inc');
        return $matches[1];
    } elseif (preg_match('/ (H[0-9]{4})( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/Huawei.php.inc');
        if (is_HuaweiKnownDevice($matches[1]))
          return $matches[1];
    }
    return false;
}

/**
 * Check if given UA is a Pocophone device.
 * @param string $ua
 * @return mixed
 * @private
 */
function _client_isPocophoneDevice($ua='') {
    if (preg_match('/; (Pocophone|Poco) ([A-Za-z0-9 ]{1,})( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Pocophone.php.inc');
        return $matches[2];
    } elseif (preg_match('/; (M2[0-1](02|03|04|07|10|12)[JK][0-9]{2}[ACPS][GI])( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Pocophone.php.inc');
        if (is_PocophoneKnownDevice(strtolower($matches[1])))
          return $matches[1];
    }
    return false;
}

/**
 * Check if given UA is a Kyocera device.
 * @param string $ua
 * @return boolean result
 * @private
 */
function _client_isKyoceraDevice($ua='') {
    if (preg_match('/ (KC-[A-Za-z0-9]*)( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/Kyocera.php.inc');
        return trim($matches[1]);
    } elseif (preg_match('/ ([0-9]{3}KC)( Build|\))/i', $ua, $matches)) {
        include_once('clientDevices/Kyocera.php.inc');
        return trim($matches[1]);
    } elseif (preg_match('/; (E6[0-9]{3}[A-Z]?)( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Kyocera.php.inc');
        if (is_KyoceraKnownDevice($matches[1]))
            return $matches[1];
    }
    return false;
}

/**
 * Check if given UA is a Ampe device.
 * @param string $ua
 * @return boolean result
 * @private
 */
function _client_isAmpeDevice($ua='') {
    if (preg_match('/ (A[0-9]{2}[A-Z]?)([\-| ][0-9A-ZA-Z ]*)?( Build|\))/Ui', $ua, $matches)) {
        include_once('clientDevices/Ampe.php.inc');
        if (is_AmpeKnownDevice($matches[1].$matches[2]))
          return $matches[1].$matches[2];
    }
    return false;
}

/**
 * Check Firefox fake user agents.
 * @param array $infos
 * @param string $ua
 * @return boolean result
 * @private
 */
function _client_checkFirefoxIsFake($infos, $ua) {
    /* Check firefox version altered by a toolbar */
    if (preg_match('#; rv:([0-9]+)([a-z0-9\.]+); WUID=(.*)\) Gecko/([a-z0-9\.]+) Firefox/([0-9]+)([a-z0-9\.]+)$#', $ua))
        return false;

    /* Check badly written rv "; rv:x.xx;" instead of "; rv:x.xx)" */
    if (preg_match('#; rv:([0-9]+)([a-z0-9\.]+);#', $ua))
        return true;

    /* Check Firefox signature */
    if (!preg_match('#; rv:([0-9]+)([a-z0-9\.]+)\) Gecko/([a-z0-9\.]+) Firefox/([0-9]+)([a-z0-9\.]+)$#', $ua, $matches))
        return false;

    /* Do not check version previous to 5, rv value and ff version don't match */
    if ((int)$matches[1] < 5 and (int)$matches[4] < 5)
        return false;

    /* Check modern rv value */
    if ((int)$matches[1] == 109 and (int)$matches[4] >= 109)
        return false;

    /* Check rv value and Firefox value */
    if ((int)$matches[1] != (int)$matches[4])
        return true;

    /* Check Gecko value */
    if ((int)$matches[1] != (int)$matches[3] and $matches[3] != '20100101')
        return true;

    /* By default, is not a fake */
    return false;
}
?>
